      PROGRAM ENERGY
C******************************************************************************
C 
C     PROGRAM for Calculation of Lattice Energy by EWALD method.
C             COPYRIGHT by T.Kamiya   1990.3.9-1990.3.19
C 
C             Coulomb    potential
C             Born-Mayer potential
C             Shell Ion model(Polarization coefficient)
C             Dispersion factor
C
C******************************************************************************
C
C     DEVICE
C            PC98
C            HITAC
C            ETA10
C
C
C     FILE
C       INPUT    FT05F001.....FILE NAME(FNAME) INPUT by Standard Input
C                FT07F001.....STRUCTURE DATA  (FNAME+'07.DAT')
C                FT01F001.....SYMMETRY  DATA  (FNAME+'01.DAT')
C                FT02F001.....ATOMIC    DATA  (FNAME+'02.DAT')
C       OUTPUT   FT06F001.....OUTPUT    DATA  OUTPUT TO Standard Output
C                FT08F001.....EXPANDED COOD.  (FNAME+'08.DAT')
C                FT09F001.....DISTANCE        (FNAME+'09.DAT')
C                FT04F001.....CRYSTAL   DATA  (FNAME+'04.DAT')
C
C       FT07F001   <<STRUCTURE DATA>>
C          TITLE(A70)
C          LATTICE CONSTANT         (LATTIC(6))      (REAL*8)
C          SPACE GROUP              (SPG)          (CHARACTER*10)
C          NUMBER OF ATOM in asymmetric unit
C                                   (DEPNUM)         (INTEGER)
C          ATOMIC STRUCTURE (I=1,...,DEPNUM)
C              ATOM KIND   OCCUPATION RATIO   X,Y,Z
C                       (ATNUM(I),OCCUP(I),X(3,I))(INTEGER,3 REAL*8)
C          EWALD PARAMETER          (ALPHA)          (REAL*8)
C              If ALPHA <0 then print calculation range.
C          Rmin(<ionic radii*2)     (IRMIN)          (REAL*8)
C          Cut level                (CEPS)           (REAL*8)
C          Distance Calculation flag(RCFLAG)         (INTEGER)
C          Distance Maximum         (RCMAX)          (REAL*8)
C          Crystal  Calculation flag(CRFLAG)         (INTEGER)
C
C       FT01F001   <<SYMMETRY DATA>>
C          SPACE GROUP              (SPG)            (CHARACTER*10)
C          CENTER SYM.              (CENT)           (INTEGER)
C          TRANS. NUMBER            (TRNUM)          (INTEGER)
C          SYM. NUMBER(INDEPENDENT) (SYMNUM)         (INTEGER)
C          TRANS. VECTOR (I=1,...,TRNUM)
C              TRS(1,I) TRS(2,I) TRS(3,I)            (REAL*8)
C          SYM. MATRIX   (I=1,...,SYMNUM)
C              R(1,1,I),...,R(3,3,I)
C              T(1,I),...,T(3,I)                     (REAL*8)
C          END
C
C       FT02F001  <<ATOMIC DATA>>
C          ATOM KIND                (ATKIND)         (INTEGER)
C          ATOM DATA(I=1,...,ATKIND)
C              ATOM NAME  MASS  CHARGE  ai  bi  POLARIZATION COEFFICIENT
C                    (ATNME(I)(A6),M(I),PZ(I)(I3),PA(I),PB(I),PP(I) (REAL*8))
C           IF ai==0.0 or aj==0.0 THEN INPUT A(I,J),B(I,J)
C          Dispersion and A(I,J)  (I=1,...,ATKIND,
C                                      J=I,...,ATKIND)
C              A(I,J)  B(I,J)  C(I,J)
C
C       PARAMETER
C             NAMAX     MAX NUMBER OF ALL ATOMS IN UNIT CELL
C             MMAX      MAX NUMBER OF MATRIX AND VECTOR
C             KMAX      MAX NUMBER OF ATOM KIND
C             NSMAX     MAX NUMBER OF SYMMETRY OPERATION
C             NTMAX     MAX NUMBER OF TRANS. OPERATION
C             
C
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(NAMAX=900,MMAX=3*NAMAX,KMAX=30,NSMAX=400,NTMAX=32)
      include 'common.prm'
      COMMON /STRING/FILE0(9),TITLE,ATNME(KMAX)
      COMMON /ATOM/PA(KMAX),PB(KMAX),PP(KMAX),PZ(KMAX)
      COMMON /ATPARA/M(KMAX),A(KMAX,KMAX),B(KMAX,KMAX),C(KMAX,KMAX),
     -               PK(KMAX,KMAX),Z(KMAX,KMAX),ATKIND
      COMMON /CELL/OCCUP(NAMAX),X(3,NAMAX),SHELID(NAMAX),
     -             ATTOT,MULT(NAMAX),DEPNUM,ATNUM(NAMAX)
      COMMON /CRYSTL/LATTIC(6),G(9),VOL,RLATIC(6),RG(9),RVOL
      COMMON /MINMAX/IRMIN,CEPS,RCMAX,RMAX,G2MAX,RDMAX,RRMAX,RPMAX,
     -               ALPHA,NRMAX(3),HGMAX(3)
      COMMON /FLAGS/RCFLAG,CRFLAG
      COMMON /SYMM/TRS(3,NTMAX),R(3,3,NSMAX),T(3,NSMAX),
     -             CENT,TRNUM,SYMNUM
C
C
C -----   STRINGS   -----
      CHARACTER*70 TITLE
C  Basic File Name
      CHARACTER*30 FILE0
C
C -----   ATOMIC  PARAMETER   -----
C  Atom Name
      CHARACTER*6  ATNME
C  Mass   ai  bi   Polarization coefficient
      REAL*8    M,PA,PB,PP
C  Charge     Number of Atom Kinds
      real*8 pz
      INTEGER ATKIND
C  Inter ionic potential parameter   Aij,bi+bj,Cij,Polarization,Coulomb
      REAL*8    A,B,C,PK,Z
C
C -----   CRYSTAL  PARAMETER   -----
C  Direct Latttice and Reciprocal Lattice parameter
      REAL*8    LATTIC,G,VOL,RLATIC,RG,RVOL
C   Total number of Atom in unit cell   Multiplicity of Independent Atom
      INTEGER ATTOT,MULT
C  Occupancy  Coordinate   EWALD parameter
      REAL*8    OCCUP,X,ALPHA
C  Number of dependent Atoms in unit cell      Atom kind number
      INTEGER DEPNUM,ATNUM
C  Identification of Shell of Core Atom
      INTEGER SHELID
C
C -----   MINIMUM and MAXIMUM PARAMETER   -----
C  Ionic Radii Minimum     Cut level     Distance Max
C  Calculation Maximum of Direct and Reciprocal space
      REAL*8    IRMIN,CEPS,RCMAX,RMAX,G2MAX,RDMAX,RRMAX,RPMAX
      INTEGER NRMAX,HGMAX
C
C -----   SYMMETRY OPERATION PARAMETER   -----
C  Number of Trans. operation    Number of Symmetry operation
      INTEGER TRNUM,SYMNUM
C  Trans. and Symmetry operation Matrix
      REAL*8    TRS,R,T
C
C -----   FLAGS   -----
C  Distance Calculation    Crystal Calculation    Center symmetry
      INTEGER RCFLAG,CRFLAG,CENT
C
C
C*****************************************************************************
C
C                 MAIN PROGRAM
C
C*****************************************************************************
      INTEGER I,ATN
      INTEGER OPENFL,CLOSFL
C  RANGE CALCULATION FLAG
      INTEGER RNGFLG
      REAL*8 DENS
      CHARACTER*10 SPG
C
C  <<TITLE PRINT>>
      WRITE(6,101) ('=',I=1,78)
  101 FORMAT(1H ,78A)
      WRITE(6,*)'          PROGRAM for Calculation of Lattice ',
     -          'Energy using EWALD method.'
      WRITE(6,*)'                       COPYRIGHT by T.Kamiya   ',
     -          '1990.3.19'
      WRITE(6,101) ('=',I=1,78)
      WRITE(6,*)
C
C  <<DATA READ>>
C
C << PC98
      Read(5,*)
      READ(5,102) FILE0(1)
      READ(5,102) FILE0(2)
      READ(5,102) FILE0(4)
      READ(5,102) FILE0(7)
      READ(5,102) FILE0(8)
      READ(5,102) FILE0(9)
  102 FORMAT(A)
      IF(OPENFL(1).EQ.0) GOTO 1000
C PC98 >>
      CALL RDFL07(SPG)
      RNGFLG=1
      IF(ALPHA.LT.0.0D0) THEN
          RNGFLG=0
          ALPHA=-ALPHA
      END IF
      CALL RDFL01(SPG)
      CALL RDFL02
C << PC98
      IF(OPENFL(2).EQ.0) GOTO 1000
C PC98 >>
      write(6,*) 'CALCRS entered'
      CALL CALCRS(CRFLAG)
      write(6,*) 'EXTCOD entered'
      CALL EXTCOD
      IF((RCFLAG.NE.0).AND.(RNGFLG.EQ.1)) THEN
          write(6,*) 'DISTAN entered'
          CALL DISTAN
      END IF
      IF(CRFLAG.EQ.1) THEN
          DENS=0.0D0
          DO 401 I=1,DEPNUM
              ATN=ATNUM(I)
              DENS=DENS+M(ATN)*DBLE(MULT(I))*OCCUP(I)
  401     CONTINUE
          WRITE(4,402) DENS/(VOL*6.0221367D-1)
  402     FORMAT(1H /1H ,'Xray Density=',F12.6,'(g/cm^3)')
      END IF
      write(6,*) 'CALRNG entered'
      CALL CALRNG
      IF(RNGFLG.EQ.1) Then
        write(6,*) 'CALENG entered'
	  CALL CALENG
      End If
C << PC98
      IF(CLOSFL().EQ.0) GOTO 1000
C PC98 >>
      WRITE(6,300)
  300 FORMAT(1H /1H ,'***************')
      WRITE(6,*)     '  PROGRAM END'
      WRITE(6,310)
  310 FORMAT(1H ,'***************')
 1000 CONTINUE
      END
C
C
C******************************************************************************
C          CALCULATION OF ENERGY
C******************************************************************************
      SUBROUTINE CALENG
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(NAMAX=900,MMAX=3*NAMAX,KMAX=30,NSMAX=400,NTMAX=32)
      include 'common.prm'
      COMMON /STRING/FILE0(9),TITLE,ATNME(KMAX)
      COMMON /ATOM/PA(KMAX),PB(KMAX),PP(KMAX),PZ(KMAX)
      COMMON /ATPARA/M(KMAX),A(KMAX,KMAX),B(KMAX,KMAX),C(KMAX,KMAX),
     -               PK(KMAX,KMAX),Z(KMAX,KMAX),ATKIND
      COMMON /CELL/OCCUP(NAMAX),X(3,NAMAX),SHELID(NAMAX),
     -             ATTOT,MULT(NAMAX),DEPNUM,ATNUM(NAMAX)
      COMMON /CRYSTL/LATTIC(6),G(9),VOL,RLATIC(6),RG(9),RVOL
      COMMON /MINMAX/IRMIN,CEPS,RCMAX,RMAX,G2MAX,RDMAX,RRMAX,RPMAX,
     -               ALPHA,NRMAX(3),HGMAX(3)
C
C
      CHARACTER*70 TITLE
      CHARACTER*30 FILE0
      CHARACTER*6  ATNME
      REAL*8    M,PA,PB,PP,pz
      INTEGER ATKIND
      REAL*8    A,B,C,PK,Z
      REAL*8    LATTIC,G,VOL,RLATIC,RG,RVOL
      INTEGER ATTOT,MULT
      REAL*8    OCCUP,X,ALPHA
      INTEGER DEPNUM,ATNUM
      INTEGER SHELID
      REAL*8    IRMIN,CEPS,RCMAX,RMAX,G2MAX,RDMAX,RRMAX,RPMAX
      INTEGER NRMAX,HGMAX
C
      INTEGER T,T1,I,N1,N2,N3,ATN,ATN1
C           COULOMB(SUM DIRECT RECIPROCAL CONST.)  REPULSION
C           DISPERSION  POLARIZATION   TOTAL
      REAL*8 UCO(NAMAX),UC1(NAMAX),UC2(NAMAX),UC3(NAMAX),UA(NAMAX),
     -       UC(NAMAX),UP(NAMAX),UT(NAMAX),um(namax)
      REAL*8 DUM,UTCO,UTC1,UTC2,UTC3,UTA,UTC,UTP,UTT,NA,utm
      REAL*8 KEXP,PI,PI2,R2PMAX,R2DMAX,R2RMAX,R2MIN,KC3,KREC,KFREC
      REAL*8 PHI,R,R2,R6,G2,AL2,EXPG,RERFC,EXPR,EXPGG2,EXPR2
      REAL*8 DX(3),DX0(3),KFC2
      REAL*8 SNPI(NAMAX),CSPI(NAMAX),FCAL,CSSUM,SNSUM,FCALF
      REAL*8 FC1(3,NAMAX),FC2(3,NAMAX),FR(3,NAMAX),fm(3,namax)
      REAL*8 FC(3,NAMAX),FT(3,NAMAX),OCC
      REAL*8 FTOT(3),GTT1I(3),F1K,KERFC,KEINV,DELTA
      REAL*8 ALR,ALR2,ALR4,ALR6,R3,R10,R42,RZ
      REAL*8 R5,R14,KF3,R54
C
      PI=3.1415926535D0
      NA=6.0220D23
      PI2=PI*2.0D0
      AL2=ALPHA*ALPHA
      KEXP=0.25D0/AL2
      KC3=-2.0D0*ALPHA/DSQRT(PI)
      KERFC=-KC3
      KREC=2.0D0*PI2/VOL*2.307100407D-18
      KFREC=PI2*KREC
      KEINV=1.0D0/1.60217733E-19
      R2DMAX=RDMAX*RDMAX
      R2RMAX=RRMAX*RRMAX
      R2PMAX=RPMAX*RPMAX
      r2mmax=rmmax*rmmax
      R2MIN=IRMIN*IRMIN
      R3=1.0D0/3.0D0
      R10=0.10D0
      R42=1.0D0/42.0D0
      R54=1.0D0/54.0D0
      R5=0.2D0
      R14=1.0D0/14.0D0
      KF3=2.0D0*AL2*KC3
C
C
      WRITE(6,100)
  100 FORMAT(1H /1H ,'<<ENERGY CALCULATION>>')
      WRITE(6,101)
  101 FORMAT(1H ,'Energies at ionic sites in asymmetric unit')
      DO 1 I=1,9
          RG(I)=RG(I)*PI2*PI2
    1 CONTINUE
      DO 2 T=1,DEPNUM
           UC1(T)=0.0D0
           UC2(T)=0.0D0
           UC3(T)=0.0D0
           UA(T)=0.0D0
           UC(T)=0.0D0
           UP(T)=0.0D0
           DO 5 I=1,3
              FC1(I,T)=0.0D0
              FC2(I,T)=0.0D0
              FR(I,T) =0.0D0
    5     CONTINUE
    2 CONTINUE
C
C  Coulomb Reciprocal Term   (UC2)
C
      DO 10 N1=-HGMAX(1),HGMAX(1)
        DO 11 N2=-HGMAX(2),HGMAX(2)
          DO 12 N3=0,HGMAX(3)
            G2=RG(1)*DBLE(N1*N1)+RG(2)*DBLE(N2*N2)+RG(3)*DBLE(N3*N3)
     -        +RG(7)*N2*N3+RG(8)*N1*N3+RG(9)*N1*N2
            IF(G2.GT.G2MAX) GOTO 12
            IF(G2.EQ.0.0D0) GOTO 12
            DO 13 T=1,ATTOT
              PHI=DBLE(N1)*X(1,T)+DBLE(N2)*X(2,T)+DBLE(N3)*X(3,T)
              PHI=PHI*PI2
              ATN=ATNUM(T)
              DUM=DBLE(PZ(ATN))*OCCUP(T)
              SNPI(T)=DSIN(PHI)*DUM
              CSPI(T)=DCOS(PHI)*DUM
   13       CONTINUE
            CSSUM=0.0D0
            SNSUM=0.0D0
            DO 15 T1=1,ATTOT
              CSSUM=CSSUM+CSPI(T1)
              SNSUM=SNSUM+SNPI(T1)
   15       CONTINUE
            EXPG=DEXP(-G2*KEXP)
            EXPGG2=EXPG/G2
            DO 14 T=1,DEPNUM
              FCAL =CSPI(T)*CSSUM+SNPI(T)*SNSUM
              FCALF=SNPI(T)*CSSUM-CSPI(T)*SNSUM
              IF(N3.NE.0) THEN
                FCAL=FCAL+FCAL
                FCALF=FCALF+FCALF
              END IF
              UC2(T)=UC2(T)+EXPGG2*FCAL
              KFC2=EXPGG2*FCALF
              FC2(1,T)=FC2(1,T)+KFC2*DBLE(N1)
              FC2(2,T)=FC2(2,T)+KFC2*DBLE(N2)
              FC2(3,T)=FC2(3,T)+KFC2*DBLE(N3)
   14       CONTINUE
C
   12     CONTINUE
   11   CONTINUE
   10 CONTINUE
      DO 16 T=1,DEPNUM
          UC2(T)=UC2(T)*KREC
          FC2(1,T)=FC2(1,T)*KFREC
          FC2(2,T)=FC2(2,T)*KFREC
          FC2(3,T)=FC2(3,T)*KFREC
   16 CONTINUE
C
C  Coulomb Direct Term
C
      DO 20 T=1,DEPNUM
        ATN =ATNUM(T)
        DO 21 T1=1,ATTOT
          ATN1=ATNUM(T1)
          OCC=OCCUP(T)*OCCUP(T1)
          DX0(1)=X(1,T1)-X(1,T)
          DX0(2)=X(2,T1)-X(2,T)
          DX0(3)=X(3,T1)-X(3,T)
          RZ=Z(ATN,ATN1)*OCC
          DELTA=-OCC
          DO 22 N1=-NRMAX(1),NRMAX(1)
            DX(1)=DX0(1)+N1
            DO 23 N2=-NRMAX(2),NRMAX(2)
              DX(2)=DX0(2)+N2
              DO 24 N3=-NRMAX(3),NRMAX(3)
                DX(3)=DX0(3)+N3
                R2=G(1)*DX(1)*DX(1)+G(2)*DX(2)*DX(2)+G(3)*DX(3)*DX(3)
     -            +G(7)*DX(2)*DX(3)+G(8)*DX(1)*DX(3)+G(9)*DX(1)*DX(2)
                R=DSQRT(R2)
                ALR=ALPHA*R
                ALR2=ALR*ALR
                GTT1I(1)=(G(1)*DX(1)+G(6)*DX(2)+G(5)*DX(3))*DELTA
                GTT1I(2)=(G(6)*DX(1)+G(2)*DX(2)+G(4)*DX(3))*DELTA
                GTT1I(3)=(G(5)*DX(1)+G(4)*DX(2)+G(3)*DX(3))*DELTA
                IF(R2.LT.R2MIN) THEN
                  ALR4=ALR2*ALR2
                  ALR6=ALR4*ALR2
                  FCAL=1.0D0-R3*ALR2+R10*ALR4-R42*ALR6
                  UC3(T)=UC3(T)+RZ*KC3*FCAL
                  F1K=-KF3*Z(ATN,ATN1)*(-R3+R5*ALR2-R14*ALR4+R54*ALR6)
                  FC1(1,T)=FC1(1,T)+F1K*GTT1I(1)
                  FC1(2,T)=FC1(2,T)+F1K*GTT1I(2)
                  FC1(3,T)=FC1(3,T)+F1K*GTT1I(3)
                  GOTO 24
                END IF
                IF(R2.GT.R2DMAX) GOTO 1100
                RERFC=DERFC(ALR)*Z(ATN,ATN1)/R
                EXPR2=DEXP(-ALR2)*Z(ATN,ATN1)
                UC1(T)=UC1(T)+RERFC*OCC
                F1K=(KERFC*EXPR2+RERFC)/R2
                FC1(1,T)=FC1(1,T)+F1K*GTT1I(1)
                FC1(2,T)=FC1(2,T)+F1K*GTT1I(2)
                FC1(3,T)=FC1(3,T)+F1K*GTT1I(3)
C
C  Repulsion Term
C
 1100           IF(R2.GT.R2RMAX) GOTO 1110
                IF(A(ATN,ATN1).EQ.0.0D0) GOTO 1110
                EXPR=A(ATN,ATN1)*DEXP(-R/B(ATN,ATN1))
                UA(T)=UA(T)+EXPR*OCC
                EXPR=EXPR/B(ATN,ATN1)/R
                FR(1,T)=FR(1,T)+EXPR*GTT1I(1)
                FR(2,T)=FR(2,T)+EXPR*GTT1I(2)
                FR(3,T)=FR(3,T)+EXPR*GTT1I(3)
C
C  Dispersion Term
C
 1110           IF(R2.GT.R2PMAX) GOTO 6120
                IF(C(ATN,ATN1).EQ.0.0D0) GOTO 6120
                R6=R2*R2*R2
                F1K=C(ATN,ATN1)/R6
                UC(T)=UC(T)+F1K*OCC
                F1K=F1K*6.0D0/R2
                FR(1,T)=FR(1,T)+F1K*GTT1I(1)
                FR(2,T)=FR(2,T)+F1K*GTT1I(2)
                FR(3,T)=FR(3,T)+F1K*GTT1I(3)
c
c  Morse potential
c
 6120           if(r2 .gt. r2mmax) goto 24
                if(Dij(atn,atn1) .eq. 0.0d0) goto 24
                expr=dexp(-Bij(atn,atn1)*(r-r0ij(atn,atn1)))
                um(t)=um(t)+Dij(atn,atn1)*(expr*expr-2.0*expr)
                f1k=-2.0*Dij(atn,atn1)*Bij(atn,atn1)*
     &                 (expr*expr+expr)/r
                fr(1,t)=fr(1,t)+f1k*gtt1i(1)
                fr(2,t)=fr(2,t)+f1k*gtt1i(2)
                fr(3,t)=fr(3,t)+f1k*gtt1i(3)
   24         CONTINUE
   23       CONTINUE
   22     CONTINUE
   21   CONTINUE
   20 CONTINUE
C
C POLARIZATION FACTOR
C
      DO 50 T=1,DEPNUM
          IF(SHELID(T).LT.T) GOTO 50
          ATN=ATNUM(T)
          T1=SHELID(T)
          ATN1=ATNUM(T1)
          DX(1)=X(1,T1)-X(1,T)
          DX(2)=X(2,T1)-X(2,T)
          DX(3)=X(3,T1)-X(3,T)
          R2=G(1)*DX(1)*DX(1)+G(2)*DX(2)*DX(2)+G(3)*DX(3)*DX(3)
     -      +G(7)*DX(2)*DX(3)+G(8)*DX(1)*DX(3)+G(9)*DX(1)*DX(2)
          OCC=OCCUP(T)*OCCUP(T1)
          F1K=PK(ATN,ATN1)*OCC
          UP(T)=0.50*F1K*R2
          UP(T1)=UP(T)
          GTT1I(1)=(G(1)*DX(1)+G(6)*DX(2)+G(5)*DX(3))*F1K
          GTT1I(2)=(G(6)*DX(1)+G(2)*DX(2)+G(4)*DX(3))*F1K
          GTT1I(3)=(G(5)*DX(1)+G(4)*DX(2)+G(3)*DX(3))*F1K
          FR(1,T) =FR(1,T) +GTT1I(1)
          FR(2,T) =FR(2,T) +GTT1I(2)
          FR(3,T) =FR(3,T) +GTT1I(3)
          FR(1,T1)=FR(1,T1)-GTT1I(1)
          FR(2,T1)=FR(2,T1)-GTT1I(2)
          FR(3,T1)=FR(3,T1)-GTT1I(3)
   50 CONTINUE
C
C SUM OF EACH ENERGY TERM
C
      UTCO=0.0D0
      UTC1=0.0D0
      UTC2=0.0D0
      UTC3=0.0D0
      UTA=0.0D0
      UTC=0.0D0
      UTP=0.0D0
      utm=0.0d0
      FTOT(1)=0.0D0
      FTOT(2)=0.0D0
      FTOT(3)=0.0D0
      DO 30 T=1,DEPNUM
        DO 31 I=1,3
              FC1(I,T)=FC1(I,T)/LATTIC(I)*1.0D+10
              FC2(I,T)=FC2(I,T)/LATTIC(I)*1.0D+10
              FC(I,T) =FC1(I,T)+FC2(I,T)
              FR(I,T) =FR(I,T) /LATTIC(I)*1.0D+10
              FT(I,T) =FC(I,T)+FR(I,T)
              FTOT(I) =FTOT(I)+FT(I,T)*MULT(T)
   31   CONTINUE
        UCO(T)=UC1(T)+UC2(T)+UC3(T)
        UT(T) =UCO(T)+UA(T)+UC(T)+UP(T)+um(t)
        UTCO=UTCO+UCO(T)*MULT(T)
        UTC1=UTC1+UC1(T)*MULT(T)
        UTC2=UTC2+UC2(T)*MULT(T)
        UTC3=UTC3+UC3(T)*MULT(T)
        UTA=UTA+UA(T)*MULT(T)
        UTC=UTC+UC(T)*MULT(T)
        UTP=UTP+UP(T)*MULT(T)
        utm=utm+um(t)*mult(t)
   30 CONTINUE
      UTCO=0.50D0*UTCO
      UTA=0.50D0*UTA
      UTC=0.50D0*UTC
      UTP=0.50D0*UTP
      utm=0.50d0*utm
      UTT=UTCO+UTA+UTC+UTP+utm
C
C WRITE TO FT06F001
C
      DO 60 T=1,DEPNUM
          ATN=ATNUM(T)
C	    Charge = DBLE(PZ(ATN))
          WRITE(6,110) T,ATNME(ATN),SHELID(T),OCCUP(T),MULT(T)
  110     FORMAT(1H ,'No.',I4,' ',A6,' SHELL=',I4,
     -               ' OCCUP=',F9.6,' MULT.=',I4)
          WRITE(6,120) UC1(T),UC2(T),UC3(T),UCO(T),UCO(T)*NA
  120     FORMAT(1H ,'     Uc1=',E14.6,' Uc2=',E14.6,' Uc3=',E14.6,
     -          /1H ,'     Ucoulomb=',E14.6,'*NA=',E14.6)
          WRITE(6,130) UA(T),UC(T),UP(T),um(t),UT(T),UT(T)*NA
  130     FORMAT(1H ,'     UA =',E14.6,' UC =',E14.6,' UP =',E14.6,
     -           ' UM=',e14.6,
     -          /1H ,'     Utotal  =',E14.6,'*NA=',E14.6)
          WRITE(6,131) UCO(T) / dble(PZ(ATN))
  131		FORMAT(1H ,'     Ucoulomb/Z =',E14.6)
          IF(SHELID(T).EQ.0) GOTO 59
          IF(SHELID(T).GT.T) GOTO 59
          ATN=SHELID(T)
          WRITE(6,140)
  140     FORMAT(1H ,'*Potential Sum of SHELL and CORE ')
          WRITE(6,120) UC1(T)+UC1(ATN),UC2(T)+UC2(ATN),
     -                 UC3(T)+UC3(ATN),UCO(T)+UCO(ATN),
     -                 (UCO(T)+UCO(ATN))*NA
          WRITE(6,130) UA(T)+UA(ATN),UC(T)+UC(ATN),UP(T)+UP(ATN),
     -                 um(t)+um(atn),     
     -                 UT(T)+UT(ATN),(UT(T)+UT(ATN))*NA
   59     WRITE(6,*) ' '
   60 CONTINUE
      WRITE(6,150)
  150 FORMAT(1H ,'**TOTAL SUM in UNIT CELL')
      WRITE(6,120) UTC1,UTC2,UTC3,UTCO,UTCO*NA
      WRITE(6,130) UTA,UTC,UTP,utm,UTT,UTT*NA
C
      WRITE(6,298)
  298 FORMAT(1H ,'<<FORCE CALCULATION>>')
      DO 70 T=1,DEPNUM
          ATN=ATNUM(T)
          WRITE(6,110) T,ATNME(ATN),SHELID(T),OCCUP(T),MULT(T)
          DO 71 I=1,3
            WRITE(6,300) I,FC1(I,T),FC2(I,T),FR(I,T)
  300       FORMAT(1H ,I4,' Fc1=',E14.6,' Fc2=',E14.6,' FR=',E14.6)
   71     CONTINUE
          DO 72 I=1,3
            WRITE(6,310) I,FC(I,T),FT(I,T)
  310       FORMAT(1H ,I4,' Fc =',E14.6,' Ftot=',E14.6)
   72     CONTINUE
          FCAL=KEINV/DBLE(PZ(ATN))
          WRITE(6,311) ((FC(I,T)*FCAL),I=1,3)
  311     FORMAT(1H ,'     E  =',3E14.6)
          IF(SHELID(T).EQ.0) GOTO 299
          IF(SHELID(T).GT.T) GOTO 299
          ATN1=ATN
          ATN=SHELID(T)
          WRITE(6,320)
  320     FORMAT(1H ,'*Force Sum of SHELL and CORE')
          DO 73 I=1,3
            WRITE(6,300) I,FC1(I,T)+FC1(I,ATN),FC2(I,T)+FC2(I,ATN),
     -                   FR(I,T)+FR(I,ATN)
   73     CONTINUE
          DO 74 I=1,3
            WRITE(6,310) I,FC(I,T)+FC(I,ATN),FT(I,T)+FT(I,ATN)
   74     CONTINUE
          T1=ATNUM(ATN)
          FCAL=KEINV/DBLE(PZ(ATN1)+PZ(T1))
          WRITE(6,311) (((FC(I,T)+FC(I,ATN))*FCAL),I=1,3)
  299     WRITE(6,*)
   70 CONTINUE
          WRITE(6,350) (FTOT(I),I=1,3)
  350     FORMAT(1H ,'Ftotal=',3E15.6)
      RETURN
      END
C
C
C
C******************************************************************************
C          GET RANGE OF CALCULATION
C******************************************************************************
C******************************************************************************
C******************************************************************************
C******************************************************************************
      SUBROUTINE CALRNG
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(NAMAX=900,MMAX=3*NAMAX,KMAX=30,NSMAX=400,NTMAX=32)
      include 'common.prm'
      COMMON /ATOM/PA(KMAX),PB(KMAX),PP(KMAX),PZ(KMAX)
      COMMON /ATPARA/M(KMAX),A(KMAX,KMAX),B(KMAX,KMAX),C(KMAX,KMAX),
     -               PK(KMAX,KMAX),Z(KMAX,KMAX),ATKIND
      COMMON /CELL/OCCUP(NAMAX),X(3,NAMAX),SHELID(NAMAX),
     -             ATTOT,MULT(NAMAX),DEPNUM,ATNUM(NAMAX)
      COMMON /CRYSTL/LATTIC(6),G(9),VOL,RLATIC(6),RG(9),RVOL
      COMMON /MINMAX/IRMIN,CEPS,RCMAX,RMAX,G2MAX,RDMAX,RRMAX,RPMAX,
     -               ALPHA,NRMAX(3),HGMAX(3)
C
C
      REAL*8    M,PA,PB,PP,pz
      INTEGER ATKIND
      REAL*8    A,B,C,PK,Z
      REAL*8    LATTIC,G,VOL,RLATIC,RG,RVOL,dijmax
      INTEGER ATTOT,MULT
      REAL*8    OCCUP,X,ALPHA,pmmax
      INTEGER DEPNUM,ATNUM
      INTEGER SHELID
      REAL*8    IRMIN,CEPS,RCMAX,RMAX,G2MAX,RDMAX,RRMAX,RPMAX
      INTEGER NRMAX,HGMAX
C
      REAL*8 MAX,MAX2,MAX3,DEGRAD,PMAX,N,PI2,LSIN(3),AIJMAX,CIJMAX
      INTEGER I,J,CALN
C
      PI2=3.1415926535D0*2.0D0
      DEGRAD=PI2/360.0D0
      WRITE(6,100)
  100 FORMAT(1H /1H ,'<<RANGE OF CALCULATION>>')
      WRITE(6,101) ALPHA
  101 FORMAT(1H ,'ALPHA=',E14.6)
      N=-DLOG10(CEPS)
      WRITE(6,110) CEPS,N
  110 FORMAT(1H ,'Cut level=',E14.6,'   N=',E14.6)
      PMAX=0.0D0
      pmmax=0.0d0
      AIJMAX=0.0D0
      CIJMAX=0.0D0
      dijmax=0.0d0
      DO 10 I=1,ATKIND
          DO 20 J=I,ATKIND
              IF(B(I,J).GT.PMAX) PMAX=B(I,J)
              if(Bij(i,j) .gt. pmmax) pmmax=Bij(i,j)
              IF(DABS(A(I,J)).GT.AIJMAX) AIJMAX=DABS(A(I,J))
              IF(DABS(C(I,J)).GT.CIJMAX) CIJMAX=DABS(C(I,J))
              if(dabs(Dij(i,j)) .gt. dijmax) dijmax=dabs(Dij(i,j))
   20     CONTINUE
   10 CONTINUE
      WRITE(6,120) PMAX
  120 FORMAT(1H ,'Pmax=',F10.6)
      RDMAX=(2.26D0+0.26D0*N)/ALPHA
      WRITE(6,130) RDMAX,DERFC(ALPHA*RDMAX)
  130 FORMAT(1H ,'Direct Term    : rmax=',E14.6,
     -      /1H ,'     erfc(alpha*rmax)=',E14.6)
      RRMAX=1.0D0+PMAX*N*DLOG(10.0D0)
      WRITE(6,140) RRMAX,DEXP(-RRMAX/PMAX)
  140 FORMAT(1H ,'Repulsive Term : rmax=',E14.6,
     -      /1H ,'      EXP(-rmax/Pmax)=',E14.6)
      RPMAX=N*DLOG(10.0D0)
      WRITE(6,150) RPMAX,1.0D0/RPMAX**6
  150 FORMAT(1H ,'Dispersion Term: rmax=',E14.6,
     -      /1H ,'             1/rmax^6=',E14.6)
      RMAX=RDMAX
      rmmax=1.0D0+1.0d0/pmmax*N*DLOG(10.0D0)
      WRITE(6,141) rmmax,DEXP(-rmmax*pmmax)
  141 FORMAT(1H ,'Morse Potential : rmax=',E14.6,
     -      /1H ,'      EXP(-rmax*Betamax)=',E14.6)
      IF((AIJMAX.NE.0.0D0).AND.(RRMAX.GT.RMAX)) RMAX=RRMAX
      IF((CIJMAX.NE.0.0D0).AND.(RPMAX.GT.RMAX)) RMAX=RPMAX
      if((Dijmax.ne.0.0d0).and.(rmmax.gt.rmax)) rmax=rmmax
      DO 30 I=1,3
          LSIN(I)=DSIN(DEGRAD*LATTIC(I+3))
   30 CONTINUE
      NRMAX(1)=INT(RMAX/DSQRT(G(1)*LSIN(2)*LSIN(3)))+1
      NRMAX(2)=INT(RMAX/DSQRT(G(2)*LSIN(1)*LSIN(3)))+1
      NRMAX(3)=INT(RMAX/DSQRT(G(3)*LSIN(1)*LSIN(2)))+1
      WRITE(6,160) RMAX,(NRMAX(I),I=1,3)
  160 FORMAT(1H ,'  Rmax=',E14.6,'  Nmax=',3I6)
      CALN=(INT(2.0D0/3.0D0*PI2*RMAX**3/VOL)+1)*ATTOT
      WRITE(6,161) CALN
  161 FORMAT(1H ,'Number of Calculation >>',I6)
      MAX =DSQRT(G(1)*LSIN(2)*LSIN(3))*DBLE(NRMAX(1))
      MAX2=DSQRT(G(2)*LSIN(1)*LSIN(3))*DBLE(NRMAX(2))
      IF(MAX2.GT.MAX) MAX=MAX2
      MAX2=DSQRT(G(3)*LSIN(1)*LSIN(2))*DBLE(NRMAX(3))
      IF(MAX2.GT.MAX) MAX=MAX2
      MAX3=DSQRT(G(1)*LSIN(2)*LSIN(3))*DBLE(NRMAX(1)-1)
      MAX2=DSQRT(G(2)*LSIN(1)*LSIN(3))*DBLE(NRMAX(2)-1)
      IF(MAX2.GT.MAX3) MAX3=MAX2
      MAX2=DSQRT(G(3)*LSIN(1)*LSIN(2))*DBLE(NRMAX(3)-1)
      IF(MAX2.GT.MAX3) MAX3=MAX2
C
      WRITE(6,162) MAX3,MAX
  162 FORMAT(1H ,'  Rmax(Calculation)=',E14.6,' -',E14.6)
      G2MAX=4.0D0*ALPHA**2*N*DLOG(10.0D0)+1.0D0
      DO 40 I=1,3
          LSIN(I)=DSIN(DEGRAD*RLATIC(I+3))
   40 CONTINUE
      HGMAX(1)=INT(DSQRT(G2MAX/(RG(1)*LSIN(2)*LSIN(3)))/PI2)+1
      HGMAX(2)=INT(DSQRT(G2MAX/(RG(2)*LSIN(1)*LSIN(3)))/PI2)+1
      HGMAX(3)=INT(DSQRT(G2MAX/(RG(3)*LSIN(1)*LSIN(2)))/PI2)+1
      WRITE(6,170) G2MAX,DEXP(-G2MAX/4.0D0/ALPHA**2),(HGMAX(I),I=1,3)
C
C  Repcirocal Term
C
  170 FORMAT(1H /1H ,'G^2max=',E14.6,
     -      /1H ,'EXP(-G^2max/4ALPHA^2)=',E14.6,
     -      /1H ,'                       Hmax=',3I6)
      CALN=INT(2.0D0/3.0D0*PI2*G2MAX**1.5/RVOL/PI2**3)+1
      WRITE(6,161) CALN
      MAX =DSQRT(RG(1)*LSIN(2)*LSIN(3))*DBLE(HGMAX(1))
      MAX2=DSQRT(RG(2)*LSIN(1)*LSIN(3))*DBLE(HGMAX(2))
      IF(MAX2.GT.MAX) MAX=MAX2
      MAX2=DSQRT(RG(3)*LSIN(1)*LSIN(2))*DBLE(HGMAX(3))
      IF(MAX2.GT.MAX) MAX=MAX2
C
      MAX3=DSQRT(RG(1)*LSIN(2)*LSIN(3))*DBLE(HGMAX(1)-1)
      MAX2=DSQRT(RG(2)*LSIN(1)*LSIN(3))*DBLE(HGMAX(2)-1)
      IF(MAX2.GT.MAX3) MAX3=MAX2
      MAX2=DSQRT(RG(3)*LSIN(1)*LSIN(2))*DBLE(HGMAX(3)-1)
      IF(MAX2.GT.MAX3) MAX3=MAX2
      WRITE(6,171) (MAX3*PI2)**2,(MAX*PI2)**2
  171 FORMAT(1H ,'G^2max(Calculation)=',E14.6,' -',E14.6)
      RETURN
      END
C
C
C******************************************************************************
C          EXTEND COORDINATES
C******************************************************************************
      SUBROUTINE EXTCOD
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(NAMAX=900,MMAX=3*NAMAX,KMAX=30,NSMAX=400,NTMAX=32)
      COMMON /STRING/FILE0(9),TITLE,ATNME(KMAX)
      COMMON /ATOM/PA(KMAX),PB(KMAX),PP(KMAX),PZ(KMAX)
      COMMON /ATPARA/M(KMAX),A(KMAX,KMAX),B(KMAX,KMAX),C(KMAX,KMAX),
     -               PK(KMAX,KMAX),Z(KMAX,KMAX),ATKIND
      COMMON /CELL/OCCUP(NAMAX),X(3,NAMAX),SHELID(NAMAX),
     -             ATTOT,MULT(NAMAX),DEPNUM,ATNUM(NAMAX)
      COMMON /CRYSTL/LATTIC(6),G(9),VOL,RLATIC(6),RG(9),RVOL
      COMMON /MINMAX/IRMIN,CEPS,RCMAX,RMAX,G2MAX,RDMAX,RRMAX,RPMAX,
     -               ALPHA,NRMAX(3),HGMAX(3)
      COMMON /FLAGS/RCFLAG,CRFLAG
      COMMON /SYMM/TRS(3,NTMAX),R(3,3,NSMAX),T(3,NSMAX),
     -             CENT,TRNUM,SYMNUM
C
C
      CHARACTER*70 TITLE
      CHARACTER*30 FILE0
      CHARACTER*6  ATNME
      REAL*8    M,PA,PB,PP,pz
      INTEGER ATKIND
      REAL*8    A,B,C,PK,Z
      REAL*8    LATTIC,G,VOL,RLATIC,RG,RVOL
      INTEGER ATTOT,MULT
      REAL*8    OCCUP,X,ALPHA
      INTEGER DEPNUM,ATNUM
      INTEGER SHELID
      REAL*8    IRMIN,CEPS,RCMAX,RMAX,G2MAX,RDMAX,RRMAX,RPMAX
      INTEGER NRMAX,HGMAX
      INTEGER TRNUM,SYMNUM
      REAL*8    TRS,R,T
      INTEGER RCFLAG,CRFLAG,CENT
C
      REAL*8 DIST
      INTEGER ATN,SFLAG,L,I,J,K,T1,CP,CP0,CP1,N(3),ID(NAMAX)
C
C
      WRITE(8,301)
  301 FORMAT(1H ,'<<EXTENDED COORDINATES>>')
      WRITE(8,302) TITLE
  302 FORMAT(1H ,'<<TITLE>>'/1H ,A70)
      N(1)=0
      N(2)=0
      N(3)=0
      CP=DEPNUM+1
      ATTOT=DEPNUM
C
      DO 10 T1=1,DEPNUM
          MULT(T1)=1
          ID(T1)=T1
          CP0=CP
C
C SYMMETRY OPERATION
C
          DO 15 K=1,SYMNUM
              DO 20 I=1,3
                  X(I,CP)=T(I,K)
                  DO 30 J=1,3
                      X(I,CP)=X(I,CP)+R(I,J,K)*X(J,T1)
   30             CONTINUE
                  CALL REDC01(X(I,CP))
   20         CONTINUE
          DIST=CALR(T1,CP,N)
          IF(DIST.LT.IRMIN) THEN
              GOTO 15
          ELSE
              DO 40 L=CP0,CP-1
                  DIST=CALR(L,CP,N)
                  IF(DIST.LT.IRMIN) GOTO 15
   40         CONTINUE
          END IF
          ATNUM(CP)=ATNUM(T1)
          OCCUP(CP)=OCCUP(T1)
          MULT(T1)=MULT(T1)+1
          ID(CP)=T1
          ATTOT=ATTOT+1
          CP=CP+1
   15     CONTINUE
          CP1=CP-1
C
C TRANS. OPERATION
C
          DO 36 L=1,TRNUM
              DO 37 I=1,3
                  X(I,CP)=X(I,T1)+TRS(I,L)
                  CALL REDC01(X(I,CP))
   37         CONTINUE
              DIST=CALR(T1,CP,N)
              IF(DIST.LT.IRMIN) GOTO 36
              DO 38 J=CP0,CP-1
                   DIST=CALR(J,CP,N)
                   IF(DIST.LT.IRMIN) GOTO 36
   38         CONTINUE
              ATNUM(CP)=ATNUM(T1)
              OCCUP(CP)=OCCUP(T1)
              MULT(T1)=MULT(T1)+1
              ID(CP)=T1
              ATTOT=ATTOT+1
              CP=CP+1
              DO 39 J=CP0,CP1
                  DO 43 I=1,3
                      X(I,CP)=X(I,J)+TRS(I,L)
                      CALL REDC01(X(I,CP))
   43             CONTINUE
                  DIST=CALR(T1,CP,N)
                  IF(DIST.LT.IRMIN) GOTO 39
                  DO 42 K=CP0,CP-1
                      DIST=CALR(K,CP,N)
                      IF(DIST.LT.IRMIN) GOTO 39
   42             CONTINUE
                  ATNUM(CP)=ATNUM(T1)
                  OCCUP(CP)=OCCUP(T1)
                  MULT(T1)=MULT(T1)+1
                  ID(CP)=T1
                  ATTOT=ATTOT+1
                  CP=CP+1
   39         CONTINUE
   36     CONTINUE
   10 CONTINUE
C
C CALCULATION OF SHELID
C
      CP=DEPNUM+1
      SFLAG=0
      DO 50 T1=1,DEPNUM
          ATN=ATNUM(T1)
          IF((PP(ATN).NE.0.0D0).AND.(SFLAG.EQ.0)) THEN
              SHELID(T1)=T1+1
              SHELID(T1+1)=T1
              SFLAG=1
          ELSE
              SFLAG=0
              CP=CP+MULT(T1)-1
              GOTO 50
          END IF
          IF(MULT(T1).EQ.1) GOTO 50
          CP1=CP+MULT(T1)-1
          DO 51 I=1,MULT(T1)-1
              SHELID(CP)=CP1
              SHELID(CP1)=CP
              CP=CP+1
              CP1=CP1+1
   51     CONTINUE
          CP=CP1
   50 CONTINUE
C
C PRINT DATA TO FT08F001
C
      WRITE(8,100) DEPNUM,ATTOT
  100 FORMAT(1H ,'Number of ATOMS in asymmetric unit>>',I4/
     -       1H ,'Total Number of ATOMS             >>',I4)
      WRITE(8,110)
  110 FORMAT(1H ,'No. ATOM      OCCUP.      X         Y         Z',
     -           '   SHELL ID   MULT.  ID')
      DO 60 I=1,DEPNUM
          L=ATNUM(I)
          WRITE(8,120) I,L,OCCUP(I),(X(J,I),J=1,3),
     -                 SHELID(I),MULT(I),ID(I)
  120     FORMAT(1H ,I4,' ',I7,F8.4,3F10.6,I5,I9,'  ',I4)
   60 CONTINUE
      DO 70 I=DEPNUM+1,ATTOT
          L=ATNUM(I)
          WRITE(8,130) I,L,OCCUP(I),(X(J,I),J=1,3),
     -                 SHELID(I),ID(I)
  130     FORMAT(1H ,I4,' ',I7,F8.4,3F10.6,I5,11X,I4)
   70 CONTINUE
      RETURN
      END
C
C
C******************************************************************************
C          REDUCTION COORDINATE IN 0 TO 1.0
C******************************************************************************
      SUBROUTINE REDC01(X)
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 X
C
   10 IF(X.LT.0.0D0) THEN
          X=X+1.0D0
          GOTO 10
      END IF
   20 IF(X.GE.1.0D0) THEN
          X=X-1.0D0
          GOTO 20
      END IF
      RETURN
      END
C
C
C******************************************************************************
C          DISTANCE(<RCMAX) CALCULATION
C******************************************************************************
      SUBROUTINE DISTAN
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(NAMAX=900,MMAX=3*NAMAX,KMAX=30,NSMAX=400,NTMAX=32)
      COMMON /STRING/FILE0(9),TITLE,ATNME(KMAX)
      COMMON /ATOM/PA(KMAX),PB(KMAX),PP(KMAX),PZ(KMAX)
      COMMON /ATPARA/M(KMAX),A(KMAX,KMAX),B(KMAX,KMAX),C(KMAX,KMAX),
     -               PK(KMAX,KMAX),Z(KMAX,KMAX),ATKIND
      COMMON /CELL/OCCUP(NAMAX),X(3,NAMAX),SHELID(NAMAX),
     -             ATTOT,MULT(NAMAX),DEPNUM,ATNUM(NAMAX)
      COMMON /CRYSTL/LATTIC(6),G(9),VOL,RLATIC(6),RG(9),RVOL
      COMMON /MINMAX/IRMIN,CEPS,RCMAX,RMAX,G2MAX,RDMAX,RRMAX,RPMAX,
     -               ALPHA,NRMAX(3),HGMAX(3)
      COMMON /FLAGS/RCFLAG,CRFLAG
C
C
      CHARACTER*70 TITLE
      CHARACTER*30 FILE0
      CHARACTER*6  ATNME
      REAL*8    M,PA,PB,PP,pz
      INTEGER ATKIND
      REAL*8    A,B,C,PK,Z
      REAL*8    LATTIC,G,VOL,RLATIC,RG,RVOL
      INTEGER ATTOT,MULT
      REAL*8    OCCUP,X,ALPHA
      INTEGER DEPNUM,ATNUM
      INTEGER SHELID
      REAL*8    IRMIN,CEPS,RCMAX,RMAX,G2MAX,RDMAX,RRMAX,RPMAX
      INTEGER NRMAX,HGMAX
      INTEGER RCFLAG,CRFLAG
C
      REAL*8 DIS
      INTEGER T,T1,N(3),N1,N2,N3,NA,NA1
      EQUIVALENCE (N(1),N1),(N(2),N2),(N(3),N3)
C
      WRITE(9,100) RCMAX
  100 FORMAT(1H ,'<<INTER ATOMIC DISTANCE (LESS THAN ',F9.4,
     -               ' A ) >>')
      WRITE(9,*) '<<TITLE>>'
      WRITE(9,201) TITLE
  201 FORMAT(1H ,A70)
      WRITE(9,101) ATTOT
  101 FORMAT(1H ,'TOTAL NUMBER OF ATOMS >>',I4)
      WRITE(9,*) ' No- (n1,n2,n3  No) ATOM - ATOM     DISTANCE(A)'
      DO 10 N1=-1,1
         DO 20 N2=-1,1
            DO 30 N3=-1,1
               DO 40 T=1,ATTOT
                  DO 50 T1=1,ATTOT
                     DIS=CALR(T,T1,N)
                     IF(DIS.GT.RCMAX) GOTO 50
                     IF((N1.EQ.0).AND.(N2.EQ.0).AND.(N3.EQ.0).AND.
     -                                          (T.EQ.T1)) GOTO 50
                     NA=ATNUM(T)
                     NA1=ATNUM(T1)
                     WRITE(9,110) T,N1,N2,N3,T1,ATNME(NA),
     -                            ATNME(NA1),DIS
  110 FORMAT(1H ,I4,' ',3I3,I4,'  ',A6,' ',A6,E14.6)
   50             CONTINUE
   40          CONTINUE
   30       CONTINUE
   20    CONTINUE
   10 CONTINUE
      RETURN
      END
C
C
C******************************************************************************
C          INTER ATOMIC DISTANCE CALCULATION(between T1,N and T)
C******************************************************************************
      DOUBLE PRECISION FUNCTION CALR(T,T1,N)
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(NAMAX=900,MMAX=3*NAMAX,KMAX=30,NSMAX=400,NTMAX=32)
      COMMON /STRING/FILE0(9),TITLE,ATNME(KMAX)
      COMMON /ATOM/PA(KMAX),PB(KMAX),PP(KMAX),PZ(KMAX)
      COMMON /ATPARA/M(KMAX),A(KMAX,KMAX),B(KMAX,KMAX),C(KMAX,KMAX),
     -               PK(KMAX,KMAX),Z(KMAX,KMAX),ATKIND
      COMMON /CELL/OCCUP(NAMAX),X(3,NAMAX),SHELID(NAMAX),
     -             ATTOT,MULT(NAMAX),DEPNUM,ATNUM(NAMAX)
      COMMON /CRYSTL/LATTIC(6),G(9),VOL,RLATIC(6),RG(9),RVOL
      COMMON /MINMAX/IRMIN,CEPS,RCMAX,RMAX,G2MAX,RDMAX,RRMAX,RPMAX,
     -               ALPHA,NRMAX(3),HGMAX(3)
      COMMON /FLAGS/RCFLAG,CRFLAG
C
C
      CHARACTER*70 TITLE
      CHARACTER*30 FILE0
      CHARACTER*6  ATNME
      REAL*8    M,PA,PB,PP,pz
      INTEGER ATKIND
      REAL*8    A,B,C,PK,Z
      REAL*8    LATTIC,G,VOL,RLATIC,RG,RVOL
      INTEGER ATTOT,MULT
      REAL*8    OCCUP,X,ALPHA
      INTEGER DEPNUM,ATNUM
      INTEGER SHELID
      REAL*8    IRMIN,CEPS,RCMAX,RMAX,G2MAX,RDMAX,RRMAX,RPMAX
      INTEGER NRMAX,HGMAX
      INTEGER RCFLAG,CRFLAG
C
      INTEGER T,T1,N(3)
C
      CALR=DSQRT(CALR2(T,T1,N))
      RETURN
      END
C
C
C******************************************************************************
C          SQUARE INTER ATOMIC DISTANCE CALCULATION(between T1,N and T)
C******************************************************************************
      DOUBLE PRECISION FUNCTION CALR2(T,T1,N)
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(NAMAX=900,MMAX=3*NAMAX,KMAX=30,NSMAX=400,NTMAX=32)
      COMMON /STRING/FILE0(9),TITLE,ATNME(KMAX)
      COMMON /ATOM/PA(KMAX),PB(KMAX),PP(KMAX),PZ(KMAX)
      COMMON /ATPARA/M(KMAX),A(KMAX,KMAX),B(KMAX,KMAX),C(KMAX,KMAX),
     -               PK(KMAX,KMAX),Z(KMAX,KMAX),ATKIND
      COMMON /CELL/OCCUP(NAMAX),X(3,NAMAX),SHELID(NAMAX),
     -             ATTOT,MULT(NAMAX),DEPNUM,ATNUM(NAMAX)
      COMMON /CRYSTL/LATTIC(6),G(9),VOL,RLATIC(6),RG(9),RVOL
      COMMON /MINMAX/IRMIN,CEPS,RCMAX,RMAX,G2MAX,RDMAX,RRMAX,RPMAX,
     -               ALPHA,NRMAX(3),HGMAX(3)
      COMMON /FLAGS/RCFLAG,CRFLAG
C
C
      CHARACTER*70 TITLE
      CHARACTER*30 FILE0
      CHARACTER*6  ATNME
      REAL*8    M,PA,PB,PP,pz
      INTEGER ATKIND
      REAL*8    A,B,C,PK,Z
      REAL*8    LATTIC,G,VOL,RLATIC,RG,RVOL
      INTEGER ATTOT,MULT
      REAL*8    OCCUP,X,ALPHA
      INTEGER DEPNUM,ATNUM
      INTEGER SHELID
      REAL*8    IRMIN,CEPS,RCMAX,RMAX,G2MAX,RDMAX,RRMAX,RPMAX
      INTEGER NRMAX,HGMAX
      INTEGER RCFLAG,CRFLAG
C
      REAL*8 DX(3)
      INTEGER I,T,T1,N(3)
C
      DO 10 I=1,3
          DX(I)=X(I,T1)+N(I)-X(I,T)
   10 CONTINUE
      CALR2=0.0D0
      DO 20 I=1,3
          CALR2=CALR2+G(I)*DX(I)*DX(I)
   20 CONTINUE
      CALR2=CALR2+G(7)*DX(2)*DX(3)+G(8)*DX(3)*DX(1)+G(9)*DX(1)*DX(2)
      RETURN
      END
C
C
C******************************************************************************
C          CRYSTAL PARAMETER CALCULATION
C******************************************************************************
      SUBROUTINE CALCRS(CRFLAG)
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(NAMAX=900,MMAX=3*NAMAX,KMAX=30,NSMAX=400,NTMAX=32)
      COMMON /STRING/FILE0(9),TITLE,ATNME(KMAX)
      COMMON /ATOM/PA(KMAX),PB(KMAX),PP(KMAX),PZ(KMAX)
      COMMON /ATPARA/M(KMAX),A(KMAX,KMAX),B(KMAX,KMAX),C(KMAX,KMAX),
     -               PK(KMAX,KMAX),Z(KMAX,KMAX),ATKIND
      COMMON /CELL/OCCUP(NAMAX),X(3,NAMAX),SHELID(NAMAX),
     -             ATTOT,MULT(NAMAX),DEPNUM,ATNUM(NAMAX)
      COMMON /CRYSTL/LATTIC(6),G(9),VOL,RLATIC(6),RG(9),RVOL
      COMMON /MINMAX/IRMIN,CEPS,RCMAX,RMAX,G2MAX,RDMAX,RRMAX,RPMAX,
     -               ALPHA,NRMAX(3),HGMAX(3)
      COMMON /SYMM/TRS(3,NTMAX),R(3,3,NSMAX),T(3,NSMAX),
     -             CENT,TRNUM,SYMNUM
C
C
      CHARACTER*70 TITLE
      CHARACTER*30 FILE0
      CHARACTER*6  ATNME
      REAL*8    M,PA,PB,PP,pz
      INTEGER ATKIND
      REAL*8    A,B,C,PK,Z
      REAL*8    LATTIC,G,VOL,RLATIC,RG,RVOL
      INTEGER ATTOT,MULT
      REAL*8    OCCUP,X,ALPHA
      INTEGER DEPNUM,ATNUM
      INTEGER SHELID
      REAL*8    IRMIN,CEPS,RCMAX,RMAX,G2MAX,RDMAX,RRMAX,RPMAX
      INTEGER NRMAX,HGMAX
      INTEGER TRNUM,SYMNUM
      REAL*8    TRS,R,T
      INTEGER CENT
C
      REAL*8 DEGRAD,RADDEG,LATK,SNLAT(3),CSLAT(3),RSNLAT(3),RCSLAT(3)
      INTEGER I,CRFLAG
C
      DEGRAD=3.1415926535D0/180D0
      RADDEG=1D0/DEGRAD
      DO 10 I=1,3
          IF(LATTIC(I+3).EQ.90D0) THEN
              SNLAT(I)=1.0D0
              CSLAT(I)=0.0D0
          ELSE
              SNLAT(I)=DSIN(DEGRAD*LATTIC(I+3))
              CSLAT(I)=DCOS(DEGRAD*LATTIC(I+3))
          END IF
          G(I)=LATTIC(I)**2
   10 CONTINUE
      VOL=1.0D0+2.0D0*CSLAT(1)*CSLAT(2)*CSLAT(3)
     -    -CSLAT(1)**2-CSLAT(2)**2-CSLAT(3)**2
      VOL=DSQRT(VOL)*LATTIC(1)*LATTIC(2)*LATTIC(3)
      RVOL=1.0D0/VOL
      G(4)=LATTIC(2)*LATTIC(3)*CSLAT(1)
      G(5)=LATTIC(3)*LATTIC(1)*CSLAT(2)
      G(6)=LATTIC(1)*LATTIC(2)*CSLAT(3)
      DO 20 I=4,6
          G(I+3)=2D0*G(I)
   20 CONTINUE
      RG(1)=LATTIC(2)*LATTIC(3)*SNLAT(1)
      RG(2)=LATTIC(3)*LATTIC(1)*SNLAT(2)
      RG(3)=LATTIC(1)*LATTIC(2)*SNLAT(3)
      DO 30 I=1,3
          RG(I)=RG(I)**2
   30 CONTINUE
      LATK=LATTIC(1)*LATTIC(2)*LATTIC(3)
      RG(4)=LATTIC(1)*LATK*(CSLAT(2)*CSLAT(3)-CSLAT(1))
      RG(5)=LATTIC(2)*LATK*(CSLAT(3)*CSLAT(1)-CSLAT(2))
      RG(6)=LATTIC(3)*LATK*(CSLAT(1)*CSLAT(2)-CSLAT(3))
      DO 35 I=1,6
          RG(I)=RG(I)/VOL**2
   35 CONTINUE
      DO 40 I=1,3
          RG(I+6)=2D0*RG(I+3)
          RLATIC(I)=DSQRT(RG(I))
   40 CONTINUE
      RCSLAT(1)=RG(4)/RLATIC(2)/RLATIC(3)
      RCSLAT(2)=RG(5)/RLATIC(3)/RLATIC(1)
      RCSLAT(3)=RG(6)/RLATIC(1)/RLATIC(2)
      DO 50 I=1,3
          IF(RCSLAT(I).LT.1.0D-5) THEN
              RLATIC(I+3)=90.0D0
              RSNLAT(I)=1.0D0
          ELSE
              RLATIC(I+3)=RADDEG*DACOS(RCSLAT(I))
              RSNLAT(I)=DSIN(DEGRAD*RLATIC(I+3))
          END IF
   50 CONTINUE
C
C  WRITE TO FT04F001 (FILE04.DAT)
      IF(CRFLAG.EQ.0) THEN
          RETURN
      END IF
  100 FORMAT(1H ,'<<CRYSTAL PARAMETER>>')
      WRITE(4,*) '<<TITLE>>'
      WRITE(4,201) TITLE
  201 FORMAT(1H ,A70)
      WRITE(4,100)
      WRITE(4,202)
  202 FORMAT(1H ,'I=4...(2,3)   I=5...(3,1)   I=6...(1,2)')
      WRITE(4,110)
  110 FORMAT(1H ,'<<Direct Lattice constant>>')
      WRITE(4,120) (LATTIC(I),I=1,6)
  120 FORMAT(1H ,6F9.4)
      WRITE(4,130) 'SIN >>',(SNLAT(I),I=1,3)
  130 FORMAT(1H ,A6,3F12.6)
      WRITE(4,130) 'COS >>',(CSLAT(I),I=1,3)
      WRITE(4,*) '<<Direct metric matrix>>'
      WRITE(4,140) G(1),G(6),G(5)
      WRITE(4,140) G(6),G(2),G(4)
      WRITE(4,140) G(5),G(4),G(3)
  140 FORMAT(1H ,3E14.6)
      WRITE(4,150) 'VOLUME=',VOL
  150 FORMAT(1H ,A8,E14.6)
      WRITE(4,160)
  160 FORMAT(1H ,'<<Reciprocal Lattice constant>>')
      WRITE(4,120) (RLATIC(I),I=1,6)
      WRITE(4,130) 'SIN >>',(RSNLAT(I),I=1,3)
      WRITE(4,130) 'COS >>',(RCSLAT(I),I=1,3)
      WRITE(4,*) '<<Reciprocal metric matrix>>'
      WRITE(4,140) RG(1),RG(6),RG(5)
      WRITE(4,140) RG(6),RG(2),RG(4)
      WRITE(4,140) RG(5),RG(4),RG(3)
      WRITE(4,150) 'VOLUME*=',RVOL
      RETURN
      END
C
C
C******************************************************************************
C          READ FILE02.DAT
C******************************************************************************
      SUBROUTINE RDFL02
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(NAMAX=900,MMAX=3*NAMAX,KMAX=30,NSMAX=400,NTMAX=32)
      include 'common.prm'
      COMMON /STRING/FILE0(9),TITLE,ATNME(KMAX)
      COMMON /ATOM/PA(KMAX),PB(KMAX),PP(KMAX),PZ(KMAX)
      COMMON /ATPARA/M(KMAX),A(KMAX,KMAX),B(KMAX,KMAX),C(KMAX,KMAX),
     -               PK(KMAX,KMAX),Z(KMAX,KMAX),ATKIND
      COMMON /CELL/OCCUP(NAMAX),X(3,NAMAX),SHELID(NAMAX),
     -             ATTOT,MULT(NAMAX),DEPNUM,ATNUM(NAMAX)
      COMMON /CRYSTL/LATTIC(6),G(9),VOL,RLATIC(6),RG(9),RVOL
      COMMON /MINMAX/IRMIN,CEPS,RCMAX,RMAX,G2MAX,RDMAX,RRMAX,RPMAX,
     -               ALPHA,NRMAX(3),HGMAX(3)
      COMMON /FLAGS/RCFLAG,CRFLAG
      COMMON /SYMM/TRS(3,NTMAX),R(3,3,NSMAX),T(3,NSMAX),
     -             CENT,TRNUM,SYMNUM
C
C
      CHARACTER*70 TITLE
      CHARACTER*30 FILE0
      CHARACTER*6  ATNME
      REAL*8    M,PA,PB,PP,PZ
      INTEGER ATKIND
      REAL*8    A,B,C,PK,Z
      REAL*8    LATTIC,G,VOL,RLATIC,RG,RVOL
      INTEGER ATTOT,MULT
      REAL*8    OCCUP,X,ALPHA
      INTEGER DEPNUM,ATNUM
      INTEGER SHELID
      REAL*8    IRMIN,CEPS,RCMAX,RMAX,G2MAX,RDMAX,RRMAX,RPMAX
      INTEGER NRMAX,HGMAX
      INTEGER TRNUM,SYMNUM
      REAL*8    TRS,R,T
      INTEGER RCFLAG,CRFLAG,CENT
C
C
      REAL*8    F0,K0
      INTEGER I,J
C
C   F0...Born-Mayer Potential CONSTANT (J/A)
      F0=6.947700141D-21
      f0m=F0
C   K0...Coulomb Potential Coefficient (J.A)  e^2/4/PI/E0*1.0E+10
      K0=2.307100407D-18
C
      WRITE(6,110)
  110 FORMAT(1H /1H ,'<<ATOM DATA>>')
      READ(2,*) ATKIND
      WRITE(6,100) ATKIND
  100 FORMAT(1H ,'Number of Atom kinds >>',I3)

      WRITE(6,200)
  200 FORMAT(1H ,'<<ATOMIC PARAMETER>>')
      WRITE(6,115)
  115 FORMAT(1H ,' No Atom    Mass  Charge    ai        bi    ',
     -           'Polarization')
      DO 10 I=1,ATKIND
          READ(2,130) ATNME(I)
  130     FORMAT(A6)
          READ(2,*) M(I),PZ(I),PA(I),PB(I),PP(I)
          WRITE(6,120) I,ATNME(I),M(I),PZ(I),PA(I),PB(I),PP(I)
  120     FORMAT(1H ,I3,1X,A6,F8.4,F8.4,F10.6,F10.6,E14.6)
   10 CONTINUE
      WRITE(6,210)
  210 FORMAT(1H ,'<<INTER ATOMIC PARAMETER>>')
      DO 20 I=1,ATKIND
          DO 30 J=I,ATKIND
              READ(2,*) A(I,J),B(I,J),C(I,J),Dij(i,j),Bij(i,j),
     &                  r0ij(i,j)
              Dij(i,j)=f0m*Dij(i,j)
              IF(PA(I)*PA(J).NE.0.0D0) THEN
                  B(I,J)=PB(I)+PB(J)
                  A(I,J)=F0*B(I,J)*DEXP((PA(I)+PA(J))/B(I,J))
              END IF
              IF(A(I,J).EQ.0.0D0) THEN
                  B(I,J)=0.01
              END IF
              Z(I,J)=K0*DBLE(PZ(I)*PZ(J))
              Z(J,I)=Z(I,J)
              A(J,I)=A(I,J)
              B(J,I)=B(I,J)
              C(J,I)=C(I,J)
              Dij(J,I)=Dij(I,J)
              Bij(J,I)=Bij(I,J)
              r0ij(J,I)=r0ij(I,J)
   30     CONTINUE
   20 CONTINUE
      WRITE(6,*) ' I  J      Z(I,J)         A(I,J)',
     -           '        B(I,J)     C(I,J)         PK(I,J)'
      DO 40 I=1,ATKIND
          DO 50 J=1,ATKIND
              WRITE(6,140) I,J,Z(I,J),A(I,J),B(I,J),
     -                     C(I,J),PK(I,J)
  140         FORMAT(1H ,2I3,2E15.6,F10.6,2E15.6)
   50     CONTINUE
   40 CONTINUE
      WRITE(6,*) ' I  J      D(I,J)       beta(I,J)',
     -           '       r*(I,J)'
      DO 41 I=1,ATKIND
          DO 51 J=1,ATKIND
              WRITE(6,140) I,J,Dij(I,J),Bij(I,J),r0ij(I,J)
   51     CONTINUE
   41 CONTINUE
      RETURN
      END
C
C
C******************************************************************************
C          READ FILE01.DAT
C******************************************************************************
      SUBROUTINE RDFL01(SPG)
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(NAMAX=900,MMAX=3*NAMAX,KMAX=30,NSMAX=400,NTMAX=32)
      COMMON /STRING/FILE0(9),TITLE,ATNME(KMAX)
      COMMON /ATOM/PA(KMAX),PB(KMAX),PP(KMAX),PZ(KMAX)
      COMMON /ATPARA/M(KMAX),A(KMAX,KMAX),B(KMAX,KMAX),C(KMAX,KMAX),
     -               PK(KMAX,KMAX),Z(KMAX,KMAX),ATKIND
      COMMON /CELL/OCCUP(NAMAX),X(3,NAMAX),SHELID(NAMAX),
     -             ATTOT,MULT(NAMAX),DEPNUM,ATNUM(NAMAX)
      COMMON /CRYSTL/LATTIC(6),G(9),VOL,RLATIC(6),RG(9),RVOL
      COMMON /FLAGS/RCFLAG,CRFLAG
      COMMON /SYMM/TRS(3,NTMAX),R(3,3,NSMAX),T(3,NSMAX),
     -             CENT,TRNUM,SYMNUM
C
C
      CHARACTER*70 TITLE
      CHARACTER*30 FILE0
      CHARACTER*6  ATNME
      REAL*8    M,PA,PB,PP,pz
      INTEGER ATKIND
      REAL*8    A,B,C,PK,Z
      REAL*8    LATTIC,G,VOL,RLATIC,RG,RVOL
      INTEGER ATTOT,MULT
      REAL*8    OCCUP,X
      INTEGER DEPNUM,ATNUM
      INTEGER SHELID
      INTEGER TRNUM,SYMNUM
      REAL*8    TRS,R,T
      INTEGER RCFLAG,CRFLAG,CENT
C
      INTEGER I,J,K
      CHARACTER*10 SPG
      CHARACTER*80 DUMMY
C
      REWIND(1)
      WRITE(6,*) '<<SYMMETRY DATA>>'
      WRITE(6,*) SPG
 9384 READ(1,200,ERR=8594) DUMMY
  200 FORMAT(A80)
      IF(DUMMY.NE.SPG) GOTO 9384
      READ(1,*) CENT,TRNUM,SYMNUM
      WRITE(6,100) CENT
  100 FORMAT(1H ,'CENTER SYMMETRY >>',I2)
      WRITE(6,110) TRNUM,SYMNUM
  110 FORMAT(1H ,'Number of Trans. Symmetry    >>',I2/
     -       1H ,'Number of Symmetry Operation >>',i2)
      WRITE(6,300)
  300 FORMAT(1H ,'<<Trans. Operation Vector>>')
      READ(1,*) ((TRS(I,J),I=1,3),J=1,TRNUM)
      DO 10 J=1,TRNUM
          WRITE(6,120) (TRS(I,J),I=1,3)
  120     FORMAT(1H ,3F10.6)
   10 CONTINUE
      WRITE(6,310)
  310 FORMAT(1H ,'<<Symmetry Operation Matrix>>')
      DO 20 K=1,SYMNUM
          READ(1,*) ((R(I,J,K),J=1,3),I=1,3)
          READ(1,*) (T(I,K),I=1,3)
          DO 30 I=1,3
              WRITE(6,130) (R(I,J,K),J=1,3),T(I,K)
  130         FORMAT(1H ,3F10.6,3X,F10.6)
   30     CONTINUE
          WRITE(6,140)
  140     FORMAT(1H )
   20 CONTINUE
      IF(CENT .EQ. 1) THEN
        DO 50 K=1,SYMNUM
          DO 51 I=1,3
            T(I,K+SYMNUM)=-T(I,K)
	    DO 52 J=1,3
	      R(I,J,K+SYMNUM)=-R(I,J,K)
   52       CONTINUE
   51     CONTINUE
   50   CONTINUE
        SYMNUM=SYMNUM*2
      ENDIF
      RETURN
 8594 WRITE(6,7300) SPG
 7300 FORMAT(1H ,'**ERROR  Can not found space group ',A10,'.')
      STOP
      END
C
C
C******************************************************************************
C          READ FILE07.DAT
C******************************************************************************
      SUBROUTINE RDFL07(SPG)
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(NAMAX=900,MMAX=3*NAMAX,KMAX=30,NSMAX=400,NTMAX=32)
      COMMON /STRING/FILE0(9),TITLE,ATNME(KMAX)
      COMMON /ATOM/PA(KMAX),PB(KMAX),PP(KMAX),PZ(KMAX)
      COMMON /ATPARA/M(KMAX),A(KMAX,KMAX),B(KMAX,KMAX),C(KMAX,KMAX),
     -               PK(KMAX,KMAX),Z(KMAX,KMAX),ATKIND
      COMMON /CELL/OCCUP(NAMAX),X(3,NAMAX),SHELID(NAMAX),
     -             ATTOT,MULT(NAMAX),DEPNUM,ATNUM(NAMAX)
      COMMON /CRYSTL/LATTIC(6),G(9),VOL,RLATIC(6),RG(9),RVOL
      COMMON /MINMAX/IRMIN,CEPS,RCMAX,RMAX,G2MAX,RDMAX,RRMAX,RPMAX,
     -               ALPHA,NRMAX(3),HGMAX(3)
      COMMON /FLAGS/RCFLAG,CRFLAG
      COMMON /SYMM/TRS(3,NTMAX),R(3,3,NSMAX),T(3,NSMAX),
     -             CENT,TRNUM,SYMNUM
C
C
      CHARACTER*70 TITLE
      CHARACTER*30 FILE0
      CHARACTER*6  ATNME
      REAL*8    M,PA,PB,PP,pz
      INTEGER ATKIND
      REAL*8    A,B,C,PK,Z
      REAL*8    LATTIC,G,VOL,RLATIC,RG,RVOL
      INTEGER ATTOT,MULT
      REAL*8    OCCUP,X,ALPHA
      INTEGER DEPNUM,ATNUM
      INTEGER SHELID
      REAL*8    IRMIN,CEPS,RCMAX,RMAX,G2MAX,RDMAX,RRMAX,RPMAX
      INTEGER NRMAX,HGMAX
      INTEGER TRNUM,SYMNUM
      REAL*8    TRS,R,T
      INTEGER RCFLAG,CRFLAG,CENT
C
      CHARACTER*10 SPG
      INTEGER I,J
	Integer nCrystal
      Integer Ida, Idb, Idc, IdAlpha, IdBeta, IdGamma
      Integer IdX(NAMAX), IdY(NAMAX), IdZ(NAMAX)
C
      WRITE(6,200)
  200 FORMAT(1H /1H ,'<<STRUCTURE DATA>>')
      Read(7,*) nCrystal
      READ(7,100) TITLE
  100 FORMAT(A70)
      WRITE(6,110) TITLE
  110 FORMAT(1H ,'<<TITLE>>'/1H ,A70)
      READ(7,*) (LATTIC(I),I=1,6)
      WRITE(6,120) (LATTIC(I),I=1,6)
  120 FORMAT(1H ,'    a=',F12.6,'     b=',F12.6,'     c=',F12.6/
     -       1H ,'alpha=',F12.6,'  beta=',f12.6,' gamma=',F12.6)
      READ(7,9674) SPG
 9674 FORMAT(A10)
      WRITE(6,*) SPG
      READ(7,*) DEPNUM
      WRITE(6,130) DEPNUM
  130 FORMAT(1H ,'Number of  ATOMS in asymmetric unit>>',I4)
      WRITE(6,*) ' ATOM OCCUPATION    X        Y         Z'
      DO 10 I=1,DEPNUM
          READ(7,*) ATNUM(I),OCCUP(I),(X(J,I),J=1,3)
          CALL REDC01(X(1,I))
          CALL REDC01(X(2,I))
          CALL REDC01(X(3,I))
          WRITE(6,140) ATNUM(I),OCCUP(I),(X(J,I),J=1,3)
  140     FORMAT(1H ,I4,4F10.6)
   10 CONTINUE
      Read(7,*) Ida, Idb, Idc, IdAlpha, IdBeta, IdGamma
      Do 11 i = 1, DepNum
	    Read(7,*) IdX(i), IdY(i), IdZ(i)
   11 Continue 
      READ(7,*) ALPHA,IRMIN,CEPS
      WRITE(6,160) ALPHA
  160 FORMAT(1H ,'EWALD PARAMETER >>',F10.6)
      WRITE(6,170) IRMIN,CEPS
  170 FORMAT(1H ,'IRMIN >>',F12.6,'   CEPS >>',G14.6)
      READ(7,*) RCFLAG,RCMAX,CRFLAG
      WRITE(6,150) RCFLAG,RCMAX,CRFLAG
  150 FORMAT(1H ,'RCFLAG >>',I2,'   RCMAX >>',F12.6,'   CRFLAG >>',I2)
      RETURN
      END
C
C << PC98
C******************************************************************************
C          OPEN FILE NAME
C                    RWFLAG     1   ..... READ FILE OPEN
C                    RWFLAG     2   ..... WRITE FILE OPEN
C******************************************************************************
      INTEGER FUNCTION OPENFL(RWFLAG)
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(NAMAX=900,MMAX=3*NAMAX,KMAX=30,NSMAX=400,NTMAX=32)
      COMMON /STRING/FILE0(9),TITLE,ATNME(KMAX)
      COMMON /FLAGS/RCFLAG,CRFLAG
C
C
      CHARACTER*70 TITLE
      CHARACTER*30 FILE0
      CHARACTER*6  ATNME
      INTEGER RCFLAG,CRFLAG
C
      INTEGER RWFLAG
      OPENFL=1
      IF(RWFLAG.EQ.1) THEN
          WRITE(6,100) 'SYMMETRY   DATA',FILE0(1)
          OPEN(1,ERR=1000,FILE=FILE0(1),STATUS='OLD')
          WRITE(6,100) 'ATOMIC     DATA',FILE0(2)
          OPEN(2,ERR=1000,FILE=FILE0(2),STATUS='OLD')
          WRITE(6,100) 'CRYSTAL    DATA',FILE0(4)
          WRITE(6,100) 'STRUCTURE  DATA',FILE0(7)
          OPEN(7,ERR=1000,FILE=FILE0(7),STATUS='OLD')
          WRITE(6,100) 'COORDINATE DATA',FILE0(8)
          OPEN(8,ERR=1000,FILE=FILE0(8))
          WRITE(6,100) 'DISTANCE   DATA',FILE0(9)
  100 FORMAT(1H ,A15,' >>',A30)
C
      ELSE
          IF(CRFLAG.NE.0) THEN
               OPEN(4,ERR=1000,FILE=FILE0(4))
          END IF
          IF(RCFLAG.NE.0) THEN
              OPEN(9,ERR=1000,FILE=FILE0(9))
          END IF
      END IF
      RETURN
C
 1000 WRITE(6,*) ' FILE OPEN ERR'
      OPENFL=0
      RETURN
      END
C
C
C******************************************************************************
C         CLOSE ALL FILE
C******************************************************************************
      INTEGER FUNCTION CLOSFL()
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(NAMAX=900,MMAX=3*NAMAX,KMAX=30,NSMAX=400,NTMAX=32)
      COMMON /FLAGS/RCFLAG,CRFLAG
C
C
      INTEGER RCFLAG,CRFLAG
C
      CLOSFL=1
          CLOSE(1,ERR=1000)
          CLOSE(2,ERR=1000)
      IF(CRFLAG.NE.0) THEN
          CLOSE(4,ERR=1000)
      END IF
          CLOSE(7,ERR=1000)
          CLOSE(8,ERR=1000)
      IF(RCFLAG.NE.0) THEN
          CLOSE(9,ERR=1000)
      END IF
      RETURN
 1000 WRITE(6,*) ' FILE CLOSE ERR'
      CLOSFL=0
      RETURN
      END
C PC98 >>
C
C
      DOUBLE PRECISION FUNCTION DERFC(X)
*****************************************************
*           complementary error function            *
*              in double precision                  *
*      COPYRIGHT : M.Mori  JUNE 30 1989  V.1        *
*****************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
*
      PARAMETER (NM = 5, NX = 13, NA = 5)
*
      DIMENSION CM(0:NM),CX(NX),CQ(NX),CA(0:NA)
*
      PARAMETER (PH = 1.7724 53850 90551 6 D0)
      PARAMETER (CMC = 2 / PH)
      PARAMETER (H = 0.5D0)
      PARAMETER (CXC = 2 * H / 3.1415 92653 58979 3 D0)
      PARAMETER (CXI = 4 / CXC)
*
C
      REAL*8 XV,Y,V
      INTEGER I
      DATA CM /
     $   0.1000000000000000D+01,
     $  -0.3333333333333333D+00,
     $   0.1000000000000000D+00,
     $  -0.2380952380952381D-01,
     $   0.4629629629629630D-02,
     $  -0.7575757575757575D-03 /
*
      DATA CX /
     $   0.7788007830714048D+00,
     $   0.3678794411714423D+00,
     $   0.1053992245618643D+00,
     $   0.1831563888873418D-01,
     $   0.1930454136227709D-02,
     $   0.1234098040866796D-03,
     $   0.4785117392129009D-05,
     $   0.1125351747192591D-06,
     $   0.1605228055185612D-08,
     $   0.1388794386496402D-10,
     $   0.7287724095819692D-13,
     $   0.2319522830243569D-15,
     $   0.4477732441718302D-18 /
      DATA CQ /
     $   0.2500000000000000D+00,
     $   0.1000000000000000D+01,
     $   0.2250000000000000D+01,
     $   0.4000000000000000D+01,
     $   0.6250000000000000D+01,
     $   0.9000000000000000D+01,
     $   0.1225000000000000D+02,
     $   0.1600000000000000D+02,
     $   0.2025000000000000D+02,
     $   0.2500000000000000D+02,
     $   0.3025000000000000D+02,
     $   0.3600000000000000D+02,
     $   0.4225000000000000D+02 /
*
      DATA CA /
     $   0.1000000000000000D+01,
     $  -0.1000000000000000D+01,
     $   0.3000000000000000D+01,
     $  -0.1500000000000000D+02,
     $   0.1050000000000000D+03,
     $  -0.9449999999999999D+03 /
*
      IF (X .LE. 0) THEN
        XV = -X
      ELSE
        XV = X
      END IF
*
      IF (XV .LE. 0.1D0) THEN
        Y = XV**2
        V = CM(NM)
        DO 10 I = NM - 1, 0, -1
          V = CM(I) + Y * V
   10   CONTINUE
        DERFC = 1 - CMC * XV * V
        RETURN
*
      ELSE IF (XV .LE. 100.0D0) THEN
        Y = XV**2
        V = 1 / (2 * Y)
        DO 20 I = 1, NX
          V = V + CX(I) / (CQ(I) + Y)
   20   CONTINUE
        V = CXC * XV * EXP(-Y) * V
        IF (XV .LT. 6.0D0) THEN
          V = V - 2 / (EXP(CXI * XV) - 1)
        END IF
        DERFC = V
*
      ELSE
        Y = 2 * XV**2
        V = CA(NA)
        DO 30 I = NA - 1, 0, -1
          V = CA(I) + Y * V
   30   CONTINUE
        V = EXP(-XV**2) / (PH * XV) * V
        DERFC = V
      END IF
*
      IF (X .GE. 0) THEN
        RETURN
      ELSE
        DERFC = 2 - DERFC
        WRITE (6,2001) X
 2001   FORMAT (' (FUNC.DERFC) invalid argument',
     $          ' X =',D23.16,' (warning only)')
        RETURN
      END IF
*
      END
