import sys
import re
import os
import glob


"""
python script converter to html
1. Title and download link are given on top
2. Comments are shown in red
3. Image files (jpg, jpeg, png) are shown at last
"""


argv = sys.argv

infile  = None
outfile = None
if len(argv) >= 2:
    infile = argv[1]

if infile is None or not os.path.isfile(infile):
    print("")
    print("Usage: python {} infile".format(sys.argv[0]))
    print("")
    exit()

# global variables
red1 = '<font color="red">'
red2 = '</font>'
blue1 = '<font color="blue">'
blue2 = '</font>'
green1 = '<font color="green">'
green2 = '</font>'


def escape(t):
    """HTML-escape the text in `t`."""
    return (t
        .replace("&", "&amp;")
        .replace("<", "&lt;")
        .replace(">", "&gt;")
        .replace("'", "&#39;")
        .replace('"', "&quot;")
        .replace(" ", "&nbsp;")
        )

outfile = re.sub(r'\.[^\.]*?$', '.html', infile)
fmask = re.sub(r'\.[^\.]*?$', '*.*', infile)
imagefiles = glob.glob(fmask)

nf = len(imagefiles)
relatedfiles = []
for i in range(nf-1, -1, -1):
    if not re.search(r'\.(jpg|jpeg|png)$', imagefiles[i]):
        if not re.search(r'\.(html|py)$', imagefiles[i]):
            relatedfiles.append(imagefiles[i])
        del imagefiles[i]

if len(imagefiles) == 0:
    imagefiles.append(re.sub(r'\.[^\.]*?$', '.jpg', infile))
imagefiles.sort()

print("in file :", infile)
print("out file:", outfile)
print("Image files ({}):".format(fmask))
for f in imagefiles:
    print("   ", f)
print("Related files ({}):".format(fmask))
for f in relatedfiles:
    print("   ", f)

fin = open(infile, 'r', encoding='utf-8')
lines = fin.readlines()
fin.close()

fout = open(outfile, 'w', encoding='utf-8')

IsComment = False
title  = 'No title'
title1 = 'No title'
for i in range(len(lines)):
    line = lines[i]
    line = line.rstrip()
    ret = re.search(r"^(.*?)('''|\"\"\")(.*)$", line)
    if ret:
        title = ''
        for j in range(i+1, len(lines)):
            if re.search(r"('''|\"\"\")", lines[j]):
                break
            title += lines[j]
            if j == i+1:
                title1 = lines[j]

        title = re.sub(r'\n', r'<br>\n', title.rstrip())
        break

title1 = title1.strip()
print("")
print("Title(single line):", title1)
print("Title(multi line) :", title)

fout.write("<html>\n" + "<head>\n")
fout.write('<meta http-equiv="Content-Language" content="ja">\n')
fout.write('<meta http-equiv="Content-Type" content="text/html; charset="utf-8">\n')
fout.write("<title>{}</title>\n".format(title1))
fout.write("</head>\n" + "<body><tt>\n")

fout.write('\n')
fout.write('<b>' + blue1 + title + blue2 + '</b><p>\n')
fout.write('Download script from <b>' + blue1 + '<a href="{}">{}</a>'.format(infile, infile) 
            + blue2 +'</b>\n')
fout.write('<br>\n')

fout.write("Related files:<br>\n")
for f in relatedfiles:
    fout.write('<a href="{}">{}</a>\n'.format(f, f))
    fout.write('<br>\n')
fout.write('<hr>\n')
fout.write('\n')


for i in range(len(lines)):
    line = lines[i]
    line = line.rstrip()
    
#    ret = re.search(r"^(\s*)(.*)$", line)
    ret = re.search(r"^(\s*#?\s*)(.*)$", line)
#    ret = re.search(r"^(#?\s*)(.*)$", line)
    if ret:
        spc  = ret[1]
        line = ret[2]
        spc  = re.sub(r' ', '&nbsp;', spc)
    else:
        spc = ''

    spc  = re.sub('\t', '&nbsp;&nbsp;&nbsp;&nbsp;', spc)

    line = spc + line

    ret = re.search(r"^(.*?)(''')(.*)$", line)
    if ret:
        if not IsComment:
            IsComment = True
            line = ret[1] + '{font_red_start}' + ret[2] + ret[3]
        else:
            IsComment = True
            line = ret[1] + ret[2] + '{font_red_end}' + ret[3]

    ret = re.search(r'^(.*?)(""")(.*)$', line)
    if ret:
        if not IsComment:
            IsComment = True
            line = ret[1] + '{font_red_start}' + ret[2] + ret[3]
        else:
            IsComment = True
            line = ret[1] + ret[2] + '{font_red_end}' + ret[3]

#    print("line: ", line)
    if re.search(r'^(.*?)(#.*?\?\?\?.*)$', line):
        line = re.sub(r'^(.*?)(#.*?\?\?\?.*)$', r'\1{font_blue_start}\2{font_blue_end}', line)
    else:
        line = re.sub(r'^(.*?)(#)(.*)$', r'\1{font_red_start}\2\3{font_red_end}', line)

    line = re.sub(r'{font_red_start}',  red1, line)
    line = re.sub(r'{font_red_end}',    red2, line)
    line = re.sub(r'{font_blue_start}', blue1, line)
    line = re.sub(r'{font_blue_end}',   blue2, line)
#    line = re.sub(r'(https?://\S+)',   '<a ref="$1">$1</a>', line)
    match = re.search(r'((http|ftp)s?://[^\s\"\'\<\>]+)', line)
    if match is not None:
        print("link = ", match[1])
        line = re.sub(r'(https?://[^\s\"\'\<\>]+)', '<a href="%s" taraget="_blank">%s</a>' % (match[1],match[1]), line)

    fout.write(line + "<br>\n")

fout.write("</tt>\n")

fout.write("\n")
fout.write("<hr>\n")
for f in imagefiles:
    fout.write('<a href="{}">\n<img border="0" src="{}" width="300">\n</a>\n'.format(f, f))
    fout.write('<br>\n')


fout.write("</body>\n")

fout.close()

print("")
