from tklib.tkobject import tkObject
import tklib.tkre as tkre
from tklib.tksci.tksci import Reduce01, Round

#from tkcrystal.tkcrystalobject import tkCrystalObject
from tklib.tkcrystal.tkatomtypeobject import SplitAtomName, Charge2f


def RoundParameter(x, tol):
    val = tol * int( (x+0.1*tol) / tol )
    return val 

def Round01(x):
    if abs(x - 1.0) < 0.0002:
        return 1.0
    if abs(x) < 0.0002:
        return 0.0
    if x > 1.0:
        return x - int(x)
    if x < 1.0:
        return x - int(x) + 1.0
    return x


#class tkAtomSiteObject(tkCrystalObject):
class tkAtomSiteObject(tkObject):
    def __init__(self, atomtype = None, label = None, name = '', charge = None, pos = [], 
            force = None, velocity = None, occ = 1.0,
            m = 1, ws = None, hydrogen = None, **args):

        self.__IsSelected = 1
        self.__Id        = 0
        self.__AtomType  = None
        self.__iAtomType = None
        self.SetAtomSite(atomtype, label, name, charge, pos, force, velocity, occ, m, ws, hydrogen)

    def __del__(self):
        pass

    def __str__(self):
        return self.ClassPath()

    def __del__(self):
        pass

    def __str__(self):
        return self.ClassPath()


    def SetAtomSite(self, atomtype = None, iatomtype = None, label = None, name = None, charge = None, pos = None, 
            force = None, velocity = None,
            occ = 1.0, m = None, ws = None, hydrogen = None):
        if atomtype is not None:
            self.SetAtomType(atomtype)
        if iatomtype is not None:
            self.SetiAtomType(iatomtype)
        if label is None:
            label = name
        self.SetLabel(label)
        self.SetAtomName(name)
        self.SetCharge(charge)
        self.SetPosition(pos)
        self.SetForce(force)
        self.SetVelocity(velocity)
        self.SetOccupancy(occ)
        self.SetMultiplicity(m)
        self.SetWycoffPosition(ws)
        self.SetHydrogen(hydrogen)

    def SetLabel(self, Label):
        if Label is None:
            self.__Label = self.AtomName
        else:
            self.__Label = Label

    def Label(self):
        return self.__Label

    def SetAtomName(self, AtomName):
        self.__AtomName = AtomName

    def AtomNameOnly(self, DelPar = 0):
        name, nametype, sitetype, charge = SplitAtomName(self.__AtomName)
#        name, nametype, sitetype, charge = self.SplitAtomName(self.__AtomName)
        if DelPar:
            return name
        return nametype

    def AtomName(self):
        return self.__AtomName

    def SetCharge(self, charge = None):
        if charge is not None:
            try:
                self.__Charge = float(charge)
            except:
                pass
        else:
            name, nametype, sitetype, charge = SplitAtomName(self.__AtomName)
#            name, nametype, sitetype, charge = self.SplitAtomName(self.__AtomName)
            self.__Charge = charge

    def Charge(self):
        if self.__Charge is not None:
            return Charge2f(self.__Charge)
        name, nametype, sitetype, charge = SplitAtomName(self.__AtomName)
#        name, nametype, sitetype, charge = self.SplitAtomName(self.__AtomName)
        if charge is None:
            return 0.0
        self.__Charge = charge
        return Charge2f(self.__Charge)

    def SetOccupancy(self, occ = 1.0):
        self.__Occupancy = occ

    def Occupancy(self):
        return self.__Occupancy

    def SetPosition(self, pos):
        self.__Pos = pos

    def Position(self, IsReduce01 = 0):
        if IsReduce01:
            x = Round01(self.__Pos[0])
            y = Round01(self.__Pos[1])
            z = Round01(self.__Pos[2])
            return [Reduce01(x), Reduce01(y), Reduce01(z)]
        return self.__Pos

    def SetForce(self, force):
        self.__Force = force

    def Force(self):
        if self.__Force is None:
            return [None, None, None]
        return self.__Force

    def SetMultiplicity(self, mult):
        self.__Multiplicity = mult

    def Multiplicity(self):
        return self.__Multiplicity

    def SetVelocity(self, v):
        self.__Velocity = v
    
    def Velocity(self):
        if self.__Velocity is None:
            return [None, None, None]
        return self.__Velocity

    def WycoffPosition(self):
        return self.__WycoffPosition

    def SetWycoffPosition(self, ws):
        self.__WycoffPosition = ws

    def Hydrogen(self):
        return self.__Hydrogen

    def SetHydrogen(self, h):
        self.__Hydrogen = h
        
    def SetiAtomType(self, i):
        self.__iAtomType = i

    def iAtomType(self):
        return self.__iAtomType

    def SetAtomType(self, at):
         self.__AtomType = at

    def AtomType(self):
        return self.__AtomType

    def SetId(self, id):
        self.SetIdAsymmetricAtomSite(id)

    def SetIdAsymmetricAtomSite(self, i):
        self.__idSite = i
        self.__IdAsymmetricAtomSite = i

    def SetIdSite(self, i):
        self.__idSite = i

    def IdSite(self):
        return self.__idSite

    def IdAsymmetricAtomSite(self):
        return self.__IdAsymmetricAtomSite

    def Id(self):
        return self.__IdAsymmetricAtomSite

    def SetIsSelected(self, IsSelected):
        self.__IsSelected = IsSelected

    def IsSelected(self):
        return self.__IsSelected

