import glob
import os

from tklib.tkprogvars import ProgramDir, DBDir, AtomDBDir
from tklib.tkutils import terminate, pint, pfloat, getarg, getintarg, getfloatarg, safe_getelement
import tklib.tkre as tkre
from tklib.tkinifile import tkIniFile

from tklib.tkobject import tkObject
#from tkcrystal.tkcrystalobject import tkCrystalObject


def Charge2f(charge):
    if type(charge) is int or type(charge) is float:
        return charge
    if charge is None or charge == '':
        return 0.0
    c = tkre.Match(r'(\d+)([\+\-])?', charge)
    if c is None or len(c) <= 1:
        pass
    else:
        if c[1] is None:
           pass
        elif c[2] == '-':
            charge = -float(c[1])
        else:
            charge = float(c[1])
    return pfloat(charge)

def SplitAtomName(atomname):
#Sr.a[b]2+ => ('Sr.a', '.a', '[b]', '2+')
    list = tkre.Match(r'([A-Z][a-z]?(\.[a-zA-Z]*)?)(\[.*\])?([\d+\-\.]*)', atomname)
#    print("a=", atomname)
#    print("  list=", list)
    name     = list[1]
    sitetype = list[2]
    charge   = list[4]
    if charge is not None and charge != '':
        c = tkre.Match(r'(\d+)([\+\-])?', charge)
        if c is None or len(c) <= 0:
            pass
        else:
            if c[1] is None:
                pass
            elif c[2] == '-':
                charge = -float(c[1])
            else:
                charge = float(c[1])

    if sitetype is None:
        nametype = name
    else:
        nametype = name + '[' + sitetype + ']'
 
    return name, nametype, sitetype, pfloat(charge)

def GetAtomName(AtomicNumber):
        nonrelpath = os.path.join(DBDir, 'nonrel')
        name = ''
        fp = open(nonrelpath)
        if not fp:
            print("")
            print("Error in tkatomtypeobject.GetAtomName(): Can not read [{}]".format(nonrelpath))
            return None
        
# H  HFS ATOM CALC.
#    1  300    1
#   1.00000   0.97804  20.00000  32.00000   0.00000
#  1.0  0.0  0.0   -0.30       1.0
        while 1:
            line     = fp.readline() # H  HFS ATOM CALC.
            nextline = fp.readline() #    1  300    1
            nextline = fp.readline() #   1.00000   0.97804  20.00000  32.00000   0.00000
            ai = nextline.split()
            n = pint(pfloat(ai[0]) + 0.00001)
            if n == AtomicNumber:
                ai = line.split()
                name = ai[0]
                break

# 次の空行までスキップ
            while 1:
                line2 = fp.readline()
                if not line2:
                    break
                
                line2 = line2.strip()
                if line2 == '':
                    break

        fp.close()
        name = name.capitalize()
        return name

def ReadAtomDB(name):
    if type(name) == int:
        name = self.GetAtomName(name)

    name, nametype, sitetype, charge = SplitAtomName(name)
    fmask = os.path.join(AtomDBDir, '*-{}.ini'.format(name))
    files = glob.glob(fmask)
    if len(files) == 0:
        print("")
        print("Error in tkcrystal.tkatomtypeobject.ReadAtomDB: ")
        print("   Can not find any file matching to [{}]".format(fmask))
        print("")
        exit()
#        return {}

#    print("AtomDB file=", files[0])
    ini = tkIniFile(files[0])
    inf = ini.ReadAll()
#    print("inf=", inf)
    list = ['AtomicRadius', 'FoundYear', 'Z', 'charge', 'mass', 
            'ai', 'bi', 'ci', 'rad', 
            'IonizationPotential', 'ElectronAffinity',  
            'BoilingPoint', 'MeltingPoint', 'Density', 'HeatCapacity']
    for key in list:
        try:
            inf[key] = float(inf[key])
        except:
            inf[key] = None
                
    return inf

def GetAtomInformation(name):
#    print("name:",name)
    if type(name) == int:
        AtomicNumber = name;
        name = GetAtomName(AtomicNumber);
        Charge = 0
    else:
        name, nametype, sitetype, charge = SplitAtomName(name)
        if name is None:
            AtomName = ''
            Charge = 0
        elif charge is None:
            AtomName = name
            Charge = 0
        else:
            AtomName = name;
            Charge = charge

    AtomName = name.capitalize()
#    print("AtomName:",AtomName)
    inf = ReadAtomDB(AtomName)
    inf["charge"] = Charge
#    print("name=", AtomName)
#    print("inf=", inf)
#    print("ZZ=", inf['Z'])
#    exit()
    
    return inf


#class tkAtomTypeObject(tkCrystalObject):
class tkAtomTypeObject(tkObject):
    def __init__(self, atomtype = None, charge = 0.0, **args):
        self.__AtomType     = None
        self.__AtomicNumber = None
        self.__Charge       = None
        self.__AtomicMass   = None
        self.SetAtomType(atomtype, charge)


#    def SplitAtomName(self, atomname):
#        return SplitAtomName(atomname)
#
    def SetAtomType(self, type = None, charge = None):
        self.__AtomType = type
        if charge is None:
            name, nametype, sitetype, charge1 = SplitAtomName(type)
#            name, nametype, sitetype, charge1 = self.SplitAtomName(type)
            if charge1 is not None:
                self.__Charge = charge1
        else:
            self.__Charge = charge

    def AtomTypeOnly(self, DelPar = 0):
        name, nametype, sitetype, charge = SplitAtomName(self.__AtomType)
#        name, nametype, sitetype, charge = self.SplitAtomName(self.__AtomType)
        if DelPar:
            return name
        return nametype

    def AtomType(self):
        return self.__AtomType

    def AtomNameOnly(self, DelPar = 0):
        return self.AtomTypeOnly(DelPar)

    def AtomName(self, DelPar = 0):
        return self.AtomType(DelPar)

    def Charge(self):
        if self.__Charge is not None:
            return Charge2f(self.__Charge)
        name, sitetype, charge = SplitAtomName(self._AtomType)
#        name, sitetype, charge = self.SplitAtomName(self._AtomType)
        if charge is None:
            return 0.0
        self.__Charge = charge
        return Charge2f(self.__Charge)

    def AtomicMass(self):
        return self.Mass()

    def Mass(self):
        if self.__AtomicMass is not None:
            return self.__AtomicMass
        name = self.AtomTypeOnly()
#        print("name=", name)
        inf = self.GetAtomInformation(name)
        try:
            return inf['mass']
        except:
            return 0.0    

    def AtomicNumber(self):
        if self.__AtomicNumber is not None:
            return self.__AtomicNumber

        name = self.AtomTypeOnly()
        inf = ReadAtomDB(name)
#        inf = self.GetAtomInformation(name)
        try:
             return inf['Z']
        except:
            return None

    def FillAtomTypeData(self):
        name = self.AtomTypeOnly()
        inf = self.GetAtomInformation(name)
        if inf['charge'] is not None:
            self.SetCharge(inf['charge'])
        if inf['mass'] is not None:
            self.SetAtomicMass(inf['mass'])

    def ReadAtomDB(self, name):
        if type(name) == int:
            name = self.GetAtomName(name)

        fmask = os.path.join(AtomDBDir, '*-{}.ini'.format(name))
        files = glob.glob(fmask)
        if len(files) == 0:
            print("")
            print("Error in tkcrystal.tkatomtypeobject.ReadAtomDB: ")
            print("   Can not find any file matching to [{}]".format(fmask))
            print("")
            return {}

#        print(f"Read DB file [{files[0]}]")
        ini = tkIniFile(files[0])
        inf = ini.ReadAll()
        list = ['AtomicRadius', 'FoundYear', 'Z', 'charge', 'mass', 
                'ai', 'bi', 'ci', 'rad', 
                'IonizationPotential', 'ElectronAffinity',  
                'BoilingPoint', 'MeltingPoint', 'Density', 'HeatCapacity']
        for key in list:
            try:
                if inf[key] is not None:
                    try:
                        inf[key] = float(inf[key])
                    except:
                        pass
            except:
                pass
                
        return inf

    def GetAtomName(self, AtomicNumber):
        return GetAtomName(AtomicNumber)

    def GetAtomInformation(self, name):
        if type(name) == int:
            AtomicNumber = name;
            name = self.GetAtomName(AtomicNumber);
            Charge = 0
        else:
            name, nametype, sitetype, charge = SplitAtomName(name)
#            name, nametype, sitetype, charge = self.SplitAtomName(name)
            if name is None:
                AtomName = ''
                Charge = 0
            elif charge is None:
                AtomName = name
                Charge = 0
            else:
                AtomName = name;
                Charge = charge

        AtomName = name.capitalize()
        inf = self.ReadAtomDB(AtomName)
        inf["charge"] = Charge
#        print("name=", AtomName)
#        print("inf=", inf)
#        print("ZZ=", inf['Z'])

        return inf
        