#%% #python3 to_stdcell.py POSCAR > refine.POSCAR.vasp
import sys
import numpy as np
import spglib
#from pymatgen import Structure
from pymatgen.core.structure import Structure

from pymatgen.core.periodic_table import get_el_sp
import collections

if len(sys.argv[1:]) == 1:
    to_primitive = True
elif len(sys.argv[1:]) == 2:
    if int(sys.argv[2]) == 0:
        to_primitive = True
    else:
        to_primitive = False

structure = Structure.from_file(sys.argv[1])

tolerance = 1.0e-3

cell = (structure.lattice.matrix, 
        structure.frac_coords, 
        structure.atomic_numbers)

lattice, scaled_positions, numbers = spglib.standardize_cell(cell, 
                                                             to_primitive=to_primitive,
                                                             no_idealize=False, 
                                                             symprec=tolerance)

print("%s %s" % (spglib.get_spacegroup(cell, symprec=tolerance), structure.formula))
print("1.000")
for i in range(3):
    for j in range(3):
                print("%20.14f" % lattice[i][j], end='')
    print('')

atomic_numbers_uniq = list(collections.OrderedDict.fromkeys(numbers))
num_species = []
for num in atomic_numbers_uniq:
    print("%s " % get_el_sp(num), end='')
    nspec = len(np.where(np.array(numbers) == num)[0])
    num_species.append(nspec)
print('')
for elem in num_species:
    print("%i " % elem, end='')
print('')
print('Direct')

for num in atomic_numbers_uniq:
    for i in range(len(scaled_positions)):
        if numbers[i] == num:
            print("%20.14f " % scaled_positions[i][0], end='')
            print("%20.14f " % scaled_positions[i][1], end='')
            print("%20.14f " % scaled_positions[i][2])
