
use strict;
use lib 'd:/Programs/Perl/lib';

use Utils;
use JFile;
use Crystal::CIF;
use Crystal::Crystal;
use Sci::Optimize;

my $CIFPath   = "aIGZO-Dens608-PBE/VCRelax/a-IGZOm184-Dens608-MDRelaxed-final.cif";
#my $EFormPath = "TotalEnergy-aIGZO-Dens608.csv";
my $EFormPath = "Summary-NonRelaxed.csv";
my $OutCSV    = "fit.csv";
my $MaxR  = 2.5; # A

my $Opt = new Optimize;

#my $Method = "PDL::Simplex";
my $Method = "Amoeba::Simplex";
#my $Method = "ModifiedNewton";
#my $Method = "LinearOptimization"; # `p[^̂݁B

my $EPS         = 1.0e-7;
my $nMaxIter    = 3000;
my $iPrintLevel = 0;

my $cif = new CIF;
$cif->Read($CIFPath);
my $Crystal = $cif->GetCCrystal();
my @latt = $Crystal->LatticeParameters();
print("Latt: ", join(', ', @latt), "\n");

$Crystal->ExpandCoordinates();
my @AtomTypeList = $Crystal->GetCAtomTypeList();
my @AtomSiteList = $Crystal->GetCAsymmetricAtomSiteList();
my @ExpandedAtomSiteList = $Crystal->GetCExpandedAtomSiteList();

my $site1 = $ExpandedAtomSiteList[0];
my $label1       = $site1->Label();
my $type1        = $site1->AtomName();
my ($x1,$y1,$z1) = $site1->Position(1);
my $site2 = $ExpandedAtomSiteList[74];
my $label2       = $site2->Label();
my $type2        = $site2->AtomName();
my ($x2,$y2,$z2) = $site2->Position(1);
#my $d = $Crystal->GetNearestInterAtomicDistance(0.313, 0.618, 0.877, 0.391, 0.461, 0.971);
my $d = $Crystal->GetNearestInterAtomicDistance($x1, $y1, $z1, $x2, $y2, $z2);
print "d=$d: ($x1,$y1,$z1)-($x2,$y2,$z2): $label1-$label2\n";

my $c = 1;
my %Coordination;
for(my $i = 0 ; $i < @ExpandedAtomSiteList ; $i++) {
	my @List;

	my $site1 = $ExpandedAtomSiteList[$i];
	my $label1       = $site1->Label();
	my $type1        = $site1->AtomName();
	my ($x1,$y1,$z1) = $site1->Position(1);
	next if($type1 !~ /^O/);

	for(my $j = 0 ; $j < @ExpandedAtomSiteList ; $j++) {
		next if($i == $j);

		my $site2 = $ExpandedAtomSiteList[$j];
		my $label2       = $site2->Label();
		my $type2        = $site2->AtomName();
		my ($x2,$y2,$z2) = $site2->Position(1);

		my $d = $Crystal->GetNearestInterAtomicDistance($x1, $y1, $z1, $x2, $y2, $z2);
		if($d < $MaxR) {
			my %hash = (
				i      => $i,
				j      => $j,
				c      => $c,
				label1 => $label1,
				type1  => $type1,
				x1     => $x1,
				y1     => $y1,
				z1     => $z1,
				label2 => $label2,
				type2  => $type2,
				x2     => $x2,
				y2     => $y2,
				z2     => $z2,
				d      => $d,
			);
#			@List = (@List, \%hash);
			push(@List, \%hash);
			$c++;
		}
	}

	@List = sort { $a->{d} <=> $b->{d}; } @List;
	my ($nIn, $nGa, $nZn, $nO) = (0, 0, 0, 0);
	foreach my $l (@List) {
		if($l->{d} < $MaxR) {
			$nIn++ if($l->{label2} =~ /In/);
			$nGa++ if($l->{label2} =~ /Ga/);
			$nZn++ if($l->{label2} =~ /Zn/);
			$nO++  if($l->{label2} =~ /O/);
		}
	}
	my $nM = $nIn + $nGa + $nZn;

	$Coordination{$label1} = \@List;
	$Coordination{"$label1-nIn"} = $nIn;
	$Coordination{"$label1-nGa"} = $nGa;
	$Coordination{"$label1-nZn"} = $nZn;
	$Coordination{"$label1-nO"} = $nO;
	$Coordination{"$label1-nM"}  = $nM;

printf "%s: nIn,Ga,Zn,M=$nIn,$nGa,$nZn,$nM\n", $label1;
}
#exit;

my $E0 = -448.109038;
my %EForm;
my $in = new JFile($EFormPath, "r");
while(my $line = $in->ReadLine()) {
	my @a = Utils::Split("\\s*,\\s*", $line);
	my ($No) = ($a[0] =~ /O(\d+)/);
#print "l: $line\n";
	next if($No <= 0);
	next if($a[2] eq '');

	$No += 0;
	$EForm{"O$No"} = $a[2] - $E0;
print "[O$No] ", $EForm{"O$No"}, " eV\n";
}
$in->Close();
#exit;

my @Guess       = (  1,   1,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0);
#my @OptId       = (  1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1,   1);
#my @OptId       = (  1,   1,   1,   0,   1,   1,   1,   0,   1,   1,   1,   0);
my @OptId       = (  1,   1,   1,   1,   1,   1,   1,   1,   0,   0,   0,   0);
my @Scale       = (0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1);
#KvȂ΁APDLDisplayFuncȍ~͎w肷Kv͂Ȃ
my ($OptVars, $MinVal) = $Opt->Optimize($Method, \@Guess, \@OptId, \@Scale, 
				$EPS, $nMaxIter, $iPrintLevel,
				\&CalS2, sub {},
				sub { Optimize::BuildDifferentialMatrixes(@_); },
				);
print "Optimized at (",join(',',@{$OptVars}),")=$MinVal\n";

my (%EOM, %kOM, %kOM2);
($EOM{In},  $EOM{Ga},  $EOM{Zn},  $EOM{O}, 
 $kOM{In},  $kOM{Ga},  $kOM{Zn},  $kOM{O},
 $kOM2{In}, $kOM2{Ga}, $kOM2{Zn}, $kOM2{O}) = @$OptVars;

my $out = new JFile($OutCSV, "w");
$out->print("Atom,EForm(obs),EForm(fit),dE(eV),nIn,nGa,nZn,nO,nM\n");
my %CN;
my %davg;
my $i = 0;
my $S2 = 0.0;
foreach my $label1 (sort keys %Coordination) {
	next if($label1 !~ /^O/);
	next if($EForm{$label1} eq '');

	$i++;
	my $pList = $Coordination{$label1};
	my $Etot = 0.0;
	foreach my $l (@$pList) {
#print "l:$label1 - $l->{label2}\n";

		my ($atom) = ($l->{label2} =~ /(\D+)/);
		my $d = $l->{d};
#		$Etot += $EOM{$atom} + $kOM{$atom} * $d + $kOM2{$atom} * $d * $d;
		$Etot += &CalBondEnergy($d, $EOM{$atom}, $kOM{$atom}, $kOM2{$atom});
		$davg{$atom} += $l->{d};
		$CN{$atom}++;
	}
	my $dE = $EForm{$label1} - $Etot;
	$S2 += $dE*$dE;
#print "$label1: E=$EForm{$label1}, $Etot, S2=$S2\n";
#print "Atom($i): [$label1]: $EForm{$label1} ";
#print "$Etot ($dE) eV\n";
my $nIn = $Coordination{"$label1-nIn"};
my $nGa = $Coordination{"$label1-nGa"};
my $nZn = $Coordination{"$label1-nZn"};
my $nO  = $Coordination{"$label1-nO"};
my $nM  = $Coordination{"$label1-nM"};
printf("Atom(%02d): [%3s]: %6.3f %6.3f (%6.3f) eV (n=$nIn,$nGa,$nZn,$nO,$nM)\n", 
			$i, $label1, $EForm{$label1}, $Etot, $dE);
	$out->print("$label1,$EForm{$label1},$Etot,$dE,$nIn,$nGa,$nZn,$nM\n");

	foreach my $l (@$pList) {
#		print "  $l->{c}: [$l->{label2}]: $l->{d}\n" if($l->{label2} !~ /^O/);
	}
}
$out->Close();

foreach my $atom qw(In Ga Zn O) {
	next if($CN{$atom} == 0);
	my $davg = $davg{$atom} / $CN{$atom};
	printf "O-$atom: d = %6.3f / %d = %8.6f A\n", $davg{$atom}, $CN{$atom}, $davg;
	my $d = $davg;
	printf "   %6.3f + %6.3f*d + %6.3f*d^2 (%8.5f) eV\n", 
			$EOM{$atom}, $kOM{$atom}, $kOM2{$atom}, &CalBondEnergy($d, $EOM{$atom}, $kOM{$atom}, $kOM2{$atom});
#	printf "   %6.3f *exp(- %6.3f*d) (%8.5f) eV\n", 
#			$EOM{$atom}, abs($kOM{$atom}), &CalBondEnergy($d, $EOM{$atom}, $kOM{$atom}, $kOM2{$atom});
}

print "i=$i\n";
print "S2=$S2\n";
print "S=", sqrt($S2/$i), " eV\n";


#rO 1.40
#rZn .6
#rIn .8
#rGa .47

#dO-In: 2.2
#dO-Ga: 1.87
#dO-Zn: 2.0

exit;

sub CalBondEnergy
{
	my ($d, $e0, $e1, $e2) = @_;
	return $e0 + $e1 * $d + $e2 * $d*$d;
#	return $e0 * exp(-abs($e1) * $d);
}

sub CalS2 {
	my ($pVars, $iPrintLevel) = @_;

	my (%EOM, %kOM, %kOM2);
	($EOM{In},  $EOM{Ga},  $EOM{Zn},  $EOM{O}, 
	 $kOM{In},  $kOM{Ga},  $kOM{Zn},  $kOM{O},
	 $kOM2{In}, $kOM2{Ga}, $kOM2{Zn}, $kOM2{O}) = @$pVars;

	my $S2 = 0.0;
	foreach my $label1 (sort keys %Coordination) {
		next if($label1 !~ /^O/);
		next if($EForm{$label1} eq '');
#print "l:$label1\n";

		my $pList = $Coordination{$label1};
		my $Etot = 0.0;
		foreach my $l (@$pList) {
#print "l:$label1 - $l->{label2}\n";
			my ($atom) = ($l->{label2} =~ /(\D+)/);
			my $d = $l->{d};
			$Etot += &CalBondEnergy($d, $EOM{$atom}, $kOM{$atom}, $kOM2{$atom});
		}
		my $dE = $EForm{$label1} - $Etot;
		$S2 += $dE*$dE;
#print "$label1: E=$EForm{$label1}, $Etot, S2=$S2\n";
	}

printf "p: %4.3f, %4.3f, %4.3f, %4.3f - %4.3f, %4.3f, %4.3f, %4.3f - %4.3f, %4.3f, %4.3f, %4.3f: %4.2f\n",
	 $EOM{In},  $EOM{Ga},  $EOM{Zn},  $EOM{O}, 
	 $kOM{In},  $kOM{Ga},  $kOM{Zn},  $kOM{O},
	 $kOM2{In}, $kOM2{Ga}, $kOM2{Zn}, $kOM2{O},$S2 ;

	return $S2;
}
