#!/bin/bash

SCRIPTDir=`pwd`/script
VASPDir=$TkPerlDir/VASP
CMD="/usr/bin/perl $VASPDir/VASP.pl"
AddMDHistoryCMD="/usr/bin/perl $SCRIPTDir/AddMDHistory.pl"
VASPCommand=./DoVASP.sh

TemplateSymDir=`pwd`/Original.sym
HistoryFile=History.csv

post=

TemplateList="300 1000 1200 1400 1500 1600 1700 1800 2000 2200 2400 2600 2800 3000 3200 3500 4000 last"

#===========================
# Calculate all
#===========================

OriginalDir=`pwd`

PrevDir=
for Template in $TemplateList
do
	WorkDir=$Template
	echo Working in $WorkDir...

	if [ -f STOP.`hostname` ]; then
		exit -1
	fi

	if [ -d $WorkDir ]; then
		echo [$WorkDir] exists.
		echo Skip.
		continue
	fi

	mkdir $WorkDir
	cp $TemplateSymDir/{hosts,INCAR,KPOINTS,POSCAR,POTCAR} $WorkDir
	if [ -f $TemplateSymDir/DoVASP.sh.$Template ]; then
		cp $TemplateSymDir/DoVASP.sh.$Template $WorkDir/DoVASP.sh
	else
		echo "sed -e \"s/{TEBEG}/$Template/g\" $TemplateSymDir/DoVASP.sh \> DoVASP.sh.temp"
		sed -e "s/{TEBEG}/$Template/g" $TemplateSymDir/DoVASP.sh > $WorkDir/DoVASP.sh.temp
		sed -e "s/{TEEND}/$Template/g" $WorkDir/DoVASP.sh.temp > $WorkDir/DoVASP.sh
		rm $WorkDir/DoVASP.sh.temp
	fi
	chmod +x $WorkDir/DoVASP.sh

	if [ "$PrevDir" = "" ]; then
		echo ''
	else
		cp $PrevDir/POSCAR $WorkDir/POSCAR
	fi

	cd $WorkDir
	echo === Working in \"`pwd`\"
	echo === Files in \"$wd\"
	ls

	echo ''
	echo === Execute [$VASPCommand]
	$VASPCommand

	echo ''
	echo === Exit to \"$OriginalDir\"
	cd $OriginalDir

	echo ''
	echo ''
	
	PrevDir=$WorkDir
done

#===========================
# Post processors
#===========================
#$SCRIPTDir/MergePositionsInCIFs.pl --Positions=$iSite 0000.last/SCF MergedCIF.cif
$SCRIPTDir/MakeSummaryCSV2.pl --Positions=$iSite --AtomType=$AtomType

rm $HistoryFile
for Template in $TemplateList
do
	$AddMDHistoryCMD $HistoryFile $Template
done
