#!/bin/bash

SCRIPTDir=`pwd`/script
VASPDir=$TkPerlDir/VASP
CMD="/usr/bin/perl $VASPDir/VASP.pl"

TemplateDir=./Original
TemplateSymDir=Original.sym
RelaxedSymPOSCAR=$TemplateSymDir/POSCAR.relaxed

iSite=16
AtomType=Se
post=-Anion
Substrate=MgO

OriginalDir=`pwd`

#===========================
# Functions
#===========================
Execute ()
{
	x=$1
	y=$2
	Atom=$3
	Substrate=$4
	NELECT=$5

	cd $OriginalDir

	WorkDir=${Atom}on$x$y-${Substrate}$post

	cif=${Atom}onO-${Substrate}.cif

	if [ -d $WorkDir ]; then
		echo "[Work]: [$WorkDir] exists. Skip."
	elif [ ! -f $TemplateDir/$cif ]; then
		echo "[Work]: [$TemplateDir/$cif] does not exist. Skip."
	else
		mkdir $WorkDir
		cp $TemplateDir/{DoVASP.sh,hosts,$cif} $WorkDir
		
		cd $WorkDir

		./DoVASP.sh init
		$CMD --Action=ModifyPOSCAR --Positions=$iSite:x:0.$x\;$iSite:y:0.$y\ $WorkDir/POSCAR
		$CMD --Action=ModifyFiles  --Function=scf --NELECT=$NELECT ./INCAR

		./DoVASP.sh
	fi
}

#===========================
# Calculate
#===========================
for Atom in F
do 
	Execute 00 00 $Atom $Substrate 72
	Execute 02 05 $Atom $Substrate 72
	Execute 00 50 $Atom $Substrate 72
	Execute 02 55 $Atom $Substrate 72
	Execute 00 25 $Atom $Substrate 72
	Execute 05 27 $Atom $Substrate 72
	Execute 25 25 $Atom $Substrate 72
	Execute 27 30 $Atom $Substrate 72
done

for Atom in Cl
do 
	Execute 00 00 $Atom $Substrate 72
	Execute 02 05 $Atom $Substrate 72
	Execute 00 50 $Atom $Substrate 72
	Execute 02 55 $Atom $Substrate 72
	Execute 00 25 $Atom $Substrate 72
	Execute 05 27 $Atom $Substrate 72
	Execute 25 25 $Atom $Substrate 72
	Execute 27 30 $Atom $Substrate 72
done

for Atom in O
do 
	Execute 00 00 $Atom $Substrate 72
	Execute 02 05 $Atom $Substrate 72
	Execute 00 50 $Atom $Substrate 72
	Execute 02 55 $Atom $Substrate 72
	Execute 00 25 $Atom $Substrate 72
	Execute 05 27 $Atom $Substrate 72
	Execute 25 25 $Atom $Substrate 72
	Execute 27 30 $Atom $Substrate 72
done

for Atom in Se
do 
	Execute 00 00 $Atom $Substrate 72
	Execute 02 05 $Atom $Substrate 72
	Execute 00 50 $Atom $Substrate 72
	Execute 02 55 $Atom $Substrate 72
	Execute 00 25 $Atom $Substrate 72
	Execute 05 27 $Atom $Substrate 72
	Execute 25 25 $Atom $Substrate 72
	Execute 27 30 $Atom $Substrate 72
done

#===========================
# Post processors
#===========================
#$SCRIPTDir/MergePositionsInCIFs.pl --Positions=$iSite 2525/SCF MergedCIF.cif
$SCRIPTDir/MakeSummaryCSV2.pl --Positions=$iSite --AtomType=$AtomType


