#!/bin/bash
# use -x option for debug

HostName=`hostname`

#==========================================
# Define programs: Usually no need to edit
#==========================================
export PerlPath=perl
export PythonPath=python

export TkProgramsDir=$DataRoot
export TkProgramsBinDir=$DataRoot/bin

export MyAppDir=$DataRoot/bin
export BaderDir=$MyAppDir/Bader
export LDDir=$MyAppDir/LD
export RamanSCDir=$MyAppDir/raman-sc
export BaderCMD="$BaderDir/bader"
export EnergyPath=$LDDir/energy

export VASPDir=$TkPerlDir/VASP
export VASPScriptDir=$VASPDir/script

export CMD="$PerlPath $VASPDir/VASP.pl"
export VASPFSCMD="$PerlPATH $VASPDir/VASPFS.pl"
export VASPLDCMD="$PerlPATH $VASPDir/VASPLD.pl"

export "PHONOPYplCMD=$PerlPATH $VASPDir/Phonopy.pl"
export "WIEN2kplCMD=$PerlPATH $TkPerlDir/WIEN2k/WIEN2k.pl"

export MakeCIFCMD="$CMD --Action=ConvCONTCARFiles"
export MakeXSFCMD="$CMD --Action=MakeRelaxXSFFile --kMagnetization=0.8 --ArrowColor=255:0:0 --ArrowRadius=0.2"
export MakeSymmetrizedCIFCMD="$WIEN2kplCMD --Action=MakeSymmetrizedStructFile --TOL=0.001 *-final.cif out.struct"

export CalIRCMD=$VASPScriptDir/CalIR.sh
export CalRamanCMD=$RamanSCDir/vasp_raman.py
export DoPhonopyTemplatePath=$VASPScriptDir/DoPhonopy-Template20180418.sh
export DoPhonopyCMD=DoPhonopy.sh

#====================================================
# Initialization: No need to edit
#====================================================
ThisScript=$0
BaseDir=`pwd`
OriginalDir=$BaseDir
WorkDir= 
ARGDIR=$BaseDir

#====================================
# Make CIF
#====================================
CIFFile=auto
# UseConventionalCell: Yes|No
UseConventionalCell=No
# DFT Functional=PAW_LDA54|PAW_PBE54|PAW_LDA52|PAW_PBE52|GGA|US|PAW|PAW_PBE
Functional=PAW_PBE54
# HybridFunctional=HF|HSE|PBE0
HybridFunctional=
# SpinPolarized: Yes|No
SpinPolarized=No

UseRecommendedPOTCAR=0
BurstPOTCAR=0

if [ "$1" != "" ]; then
	CIFFile=$1
fi

AdditionalParameters="--Param:LREAL=Auto"
#MPIParams="--Param:NPAR=$NPAR --Param:KPAR=$KPAR --Param:LPLANE=.TRUE."
MPIParams=


CMDMakeCIF="$CMD --Action=MakeINCAR --Functional=$Functional \
	--UseConventionalCell=$UseConventionalCell \
	--UseRecommendedPOTCAR=$UseRecommendedPOTCAR --Function=scf \
	--HybridFunctional=$HybridFunctional \
	--BurstPOTCAR=$BurstPOTCAR \
	$MPIParams  $AdditionalParameters \
	--SpinPolarized=$SpinPolarized $CIFFile ./"

echo ' '
echo '================================================================'
echo "  Make Input Files from CIF [$CIFFile]"
echo '================================================================'
if [ "$CIFFile" = "auto" -o -f $CIFFile ]; then
	echo
else
	echo Error: [$CIFFile] does not exist.
	exit
fi

echo run [$CMDMakeCIF]
$CMDMakeCIF
cp POTCAR POTCAR.DFT
