#!/bin/bash
# use -x option for debug

#==========================================
# Prameters
#==========================================

script_dir=${0%/*}
if [ -e $script_dir/vasp_config.sh ]; then
	. $script_dir/vasp_config.sh
elif [ -e $DataRoot/scripts/vasp_config.sh ]; then
	. $DataRoot/scripts/vasp_config.sh
else
	echo 
	echo *** ERROR: Can not find [$script_dir/vasp_config.sh] nor [$DataRoot/scripts/vasp_config.sh]
	echo
	exit
fi
if [ -e $script_dir/vasp_params.sh ]; then
	. $script_dir/vasp_params.sh
fi


# NPAR must be numberof nodes
NPARPhonon=$NUM_CPU
KPARPhonon=1

#Precision=Accurate
ADDGRIDPhonon=.TRUE.
NWRITEPhonon=3
# IBRION=5/7 are recommended, but 6/8 may work
IBRIONPhonon=8
NSWPhonon=1

LEPSILONPhonon=.TRUE.
LRPAPhonon=.FALSE.
CalOpticsPhonon=.FALSE.
CSHIFTPhonon=0.1

PrecisionPhonon=$PrecisionCommon
#PrecisionPhonon=Norm
EDIFFPhonon=$EDIFFCommon
EDIFFGPhonon=$EDIFFGCommon
ALGOPhonon=$ALGOCommon
GammaOnlyPhonon=$GammaOnlyCommon
aKProductPhonon=$aKProductCommon
#aKProductPhonon=2.0
#aKProductPhonon=1.5
#aKProductPhonon=fix:8/2,8/2,6/3
ISMEARPhonon=$ISMEARCommon
#ISMEARPhonon=-5
SIGMAPhonon=$SIGMACommon

NKREDPhonon=$NKREDCommon
HybridALGOPhonon=$HybridALGOCommon
ISMEARHybridFunctionalPhonon=$ISMEARHybridFunctionalCommon
SIGMAHybridFunctionalPhonon=$SIGMAHybridFunctionalCommon

# ISYM=0 for IR
KeepSymmetryPhonon=1

LELFPhonon=.TRUE.

LREALPhonon=Auto

#====================================================
# Phonon command
#====================================================
MPIParamsPhonon="--Param:NPAR=$NPARPhonon --Param:KPAR=$KPARPhonon --Param:LPLANE=.TRUE."
#MPIParamsPhonon=$MPIParamsCommon

# For phonon, use LREAL=.FALSE. and large ENCUT like 500 eV
#AdditionalParametersPhonon=$AdditionalParametersCommon
#AdditionalParametersPhonon="--Param:LREAL=Auto"
AdditionalParametersPhonon="--Param:LREAL=$LREALPhonon --Param::LWANNIER90_RUN=.FALSE."
#AdditionalParametersPhonon="--Param:LREAL=Auto --NBANDS=x2"
#AdditionalParametersPhonon="--Param:LREAL=.FALSE. --NBANDS=64 --Param:SYMPREC=1e-4";
#AdditionalParametersPhonon="--Param:LREAL=Auto --NBANDS=64 --ENCUT=500"
#AdditionalParametersPhonon="--Param:LREAL=.FALSE. --ENCUT=500"

CommonParametersPhonon="--SpinOrbit=$SpinOrbit --NonCollinear=$NonCollinear --LMAXMIX=$LMAXMIX \
	 --SpinPolarized=$SpinPolarized"
HFParametersPhonon="--HybridFunctional=$HybridFunctional \
	 --NKRED=$NKREDPhonon \
	 --ISMEARHybridFunctional=$ISMEARHybridFunctionalPhonon \
	 --SIGMAHybridFunctional=$SIGMAHybridFunctionalPhonon"
PrecisionParametersPhonon="--Precision=$PrecisionPhonon --Param:EDIFF=$EDIFFPhonon --Param:EDIFFG=$EDIFFGPhonon \
	 --Param:ISMEAR=$ISMEARPhonon --Param:SIGMA=$SIGMAPhonon \
	 --Param:ALGO=$ALGOPhonon \
	 --GammaOnly=$GammaOnlyPhonon --aKProduct=$aKProductPhonon"

CMDPHONON="$CMD --Action=ModifyFiles --Function=phonon \
	$MPIParamsPhonon \
	$CommonParametersPhonon \
	$HFParametersPhonon \
	$PrecisionParameters \
	$AdditionalParametersPhonon \
	--KeepSymmetry=$KeepSymmetryPhonon \
	--Param:ADDGRID=$ADDGRIDPhonon \
	--Param:NWRITE=$NWRITEPhonon --Param:NSW=$NSWPhonon \
	--CalOptics=$CalOpticsPhonon --CSHIFT=$CSHIFTPhonon \
	--LRPA=$LRPAPhonon \
	--Param:LELF=$LELFPhonon \
	$ARG1"


#====================================
# Phonon
#====================================
echo ' '
echo '================================================================'
echo "  Modify input files for Phonon"
echo '================================================================'

echo Run [$CMDPHONON]
$CMDPHONON


echo 
echo ======================================================================
echo $0 finished.
echo ======================================================================
echo 
