#!/bin/bash
# use -x option for debug

#==========================================
# Prameters
#==========================================

script_dir=${0%/*}
if [ -e $script_dir/vasp_config.sh ]; then
	. $script_dir/vasp_config.sh
elif [ -e $DataRoot/scripts/vasp_config.sh ]; then
	. $DataRoot/scripts/vasp_config.sh
else
	echo 
	echo *** ERROR: Can not find [$script_dir/vasp_config.sh] nor [$DataRoot/scripts/vasp_config.sh]
	echo
	exit
fi
if [ -e $script_dir/vasp_params.sh ]; then
	. $script_dir/vasp_params.sh
fi


#RelaxMode=vc-relax|relax
RelaxMode=vc-relax

#PrecisionRelax1=Accurate
PrecisionRelax1=Norm
EDIFFRelax1=3.0e-4
EDIFFGRelax1=5.0e-3
ALGORelax1=$ALGOCommon
GammaOnlyRelax1=0
#aKProductRelax1=2.5
#aKProductRelax1=2.0
aKProductRelax1=1.5
#aKProductRelax1=fix:8/2,8/2,6/3
ISMEARRelax1=0
#ISMEARRelax1=-5
SIGMARelax1=0.1

NKREDRelax1=$NKREDCommon
HybridALGORelax1=$HybridALGOCommon
ISMEARHybridFunctionalRelax1=$ISMEARHybridFunctionalCommon
SIGMAHybridFunctionalRelax1=$SIGMAHybridFunctionalCommon

#KeepSymmetryRelax1=0
KeepSymmetryRelax1=1

NSWRelax1=300
POTIMRelax1=$POTIMCommon

LELFRelax1=.FALSE.

#====================================================
# Relax command
#====================================================
#MPIParamsRelax1="--Param:NPAR=$NPAR --Param:KPAR=$KPAR --Param:LPLANE=.TRUE."
MPIParamsRelax1=$MPIParamsCommon

# For phonon, use LREAL=.FALSE. and large ENCUT like 500 eV
# For usual DFT use default for ENCUT (comment out) and LREAL=Auto
AdditionalParametersRelax1=$AdditionalParametersCommon
#AdditionalParametersRelax1="--Param:LREAL=Auto"
#AdditionalParametersRelax1="--Param:LREAL=Auto --Param::LWANNIER90_RUN=.FALSE."
#AdditionalParametersRelax1="--Param:LREAL=Auto --NBANDS=x2"
#AdditionalParametersRelax1="--Param:LREAL=.FALSE. --NBANDS=64 --Param:SYMPREC=1e-4";
#AdditionalParametersRelax1="--Param:LREAL=Auto --NBANDS=64 --ENCUT=500"
#AdditionalParametersRelax1="--Param:LREAL=.FALSE. --ENCUT=500"

CommonParametersRelax1="--SpinOrbit=$SpinOrbitForIonDynamics --NonCollinear=$NonCollinear --LMAXMIX=$LMAXMIX \
	--SpinPolarized=$SpinPolarized"
HFParametersRelax1="--HybridFunctional=$HybridFunctional \
	--NKRED=$NKREDRelax1 \
	--ISMEARHybridFunctional=$ISMEARHybridFunctionalRelax1 \
	--SIGMAHybridFunctional=$SIGMAHybridFunctionalRelax1"
PrecisionParametersRelax1="--Precision=$PrecisionRelax1 --Param:EDIFF=$EDIFFRelax1 --Param:EDIFFG=$EDIFFGRelax1 \
	--Param:ISMEAR=$ISMEARRelax1 --Param:SIGMA=$SIGMARelax1 \
	--Param:ALGO=$ALGORelax1 --Param:LELF=$LELFRelax1 \
	--GammaOnly=$GammaOnlyRelax1 --aKProduct=$aKProductRelax1"

CMDVCRelax="$CMD --Action=ModifyFiles --Function=$RelaxMode \
	$MPIParamsRelax1 \
	$CommonParametersRelax1 \
	$HFParametersRelax1 \
	$PrecisionParametersRelax1 \
	$AdditionalParametersRelax1 \
	--KeepSymmetry=$KeepSymmetryRelax1 \
	--PStress=$PStressRelax1 \
	--Param:NSW=$NSWRelax1 \
	--Param:POTIM=$POTIMRelax1 \
	$ARG1"


#====================================
# Relax
#====================================
echo ' '
echo '================================================================'
echo "  Modify input files for Relax1: [$CMDVCRelax]"
echo '================================================================'

$CMDVCRelax

if [ "$UseDBRWIGs" = "1" ]; then
	$CMD --Action=ChangeRWIGS --RWIGSDB=$RWIGSDB . INCAR.RWIGS
	cp INCAR INCAR.prev
	cp INCAR.RWIGS INCAR
fi

echo 
echo ======================================================================
echo vasp_modify_relax1.sh finished.
echo ======================================================================
echo 
