#!/bin/bash
# use -x option for debug


# Save current directory to Basedir and basedir variables
# In some environment BaseDir is given by the parent process such as qvasp/job
if [ "$BaseDir" == "" ]; then
	basedir=`pwd`
	BaseDir=$basedir
else
	basedir=$BaseDir
fi

echo
echo ==================================
echo  $0
echo ==================================
echo
echo ==================================
echo Read external configuration files
echo ==================================
if [ -e $BaseDir/vasp_config.sh ]; then
	echo [. $BaseDir/vasp_config.sh]
	. $BaseDir/vasp_config.sh
elif [ -e $DataRoot/scripts/vasp_config.sh ]; then
	echo [. $DataRoot/scripts/vasp_config.sh]
	. $DataRoot/scripts/vasp_config.sh
else
	echo 
	echo *** ERRROR: Can not find [$BaseDir/vasp_config.sh] nor [$DataRoot/scripts/vasp_config.sh]
	echo
	exit
fi
if [ -e $BaseDir/vasp_params.sh ]; then
	echo [. $BaseDir/vasp_params.sh]
	. $BaseDir/vasp_params.sh
elif [ -e $BaseDir/vasp_params.sh ]; then
	echo [. $BaseDir/vasp_params.sh]
	. $DataRBaseDir/vasp_params.sh
else
	echo 
	echo *** ERRROR: Can not find [$DataRBaseDir/vasp_params.sh]
	echo
	exit
fi

echo ==================================
echo


#======================================================
# Control parameters
#======================================================
# In some environment BaseDir is given by the parent process such as qvasp/jo
if [ "$VASPRunCMD" == "" ]; then
	NUM_CPU=16
	VASPRunCMD="mpirun -np $NUM_CPU vasp_std"
fi

SkipIfDirectoryExists=1

do_md=0
  md_dir="00-md"

do_relax1=1
  relax1_dir="10-relax"

do_relax2=1
  relax2_dir="11-relax"

do_scf=1
  scf_dir="20-scf"

do_hfscf=0
  hfscf_dir="21-hfscf"
  if [ "$do_hfscf" == "1" ]; then
    do_scf=1
  fi

do_dos=1
  dos_dir="30-dos"

do_band=1
  band_dir="40-band"

# Not working yet
do_electrondensity=0
  electrondensity_dir="500-edensity"

# Not working yet
do_fermisurface=0
  fermisurface_dir="510-fermisurface"

do_phonon=1
  phonon_dir="520-phonon"

# Not working yet
do_phonopy=0
  phonopy_dir="530-phonopy"
# Note: Non-Analytical Term Correction requires do_phonon=1
  do_phonopy_dos=1
  do_phonopy_pdos=1
  do_phonopy_band=1
  do_phonopy_thermalproperties=1

  export "PHONOPYplCMD=$PerlPATH $VASPDir/Phonopy.pl"
  DoPhonopyTemplatePath=$VASPScriptDir/DoPhonopy-Template20180418.sh
  DoPhonopyCMD=DoPhonopy.sh

do_ir=1
  ir_dir="540-ir"
  if [ "$do_ir" == "1" ]; then
    do_phonon=1
  fi

# Not working yet
do_raman=1
  raman_dir="550-raman"
# Note: do_ir_raman requires do_phonon=1
  if [ "$do_raman" == "1" ]; then
    do_phonon=1
  fi

echo ' '
echo '================================================================'
echo "  Run VASP processes"
echo '================================================================'

echo ' '
echo '================================================================'
echo "  Backup initial files to original"
echo '================================================================'

mkdir original
cp *.cif *.sh qvasp vasp-info *.klist *.KPOINTS host* INCAR POSCAR POTCAR KPOINTS original


if [ "$do_md" = "1" ]; then
	echo ' '
	echo '================================================================'
	echo "  Run MD"
	echo '================================================================'

	workdir=$md_dir

	if [ "$SkipIfDirectoryExists" = "1" -a -d $workdir ]; then
		echo "Dir [$workdir] exists."
		echo "Skip."
	else
		echo ' '
		echo '================================================================'
		echo "  MD run"
		echo '================================================================'
		echo 

		echo 
		echo == Change dir  to [$basedir]
		cd $basedir

		echo == Make dir [$workdir]

		mkdir $workdir

		echo 
		echo == cp files to [$workdir]
		cp ./{INCAR,POSCAR,POTCAR,KPOINTS} $workdir
#		cp ./{*.sh,INCAR,POSCAR,POTCAR,KPOINTS} $workdir

		echo 
		echo == Change dir to [$workdir]
		cd $workdir

		echo
		echo == Run [../vasp_modify_md.sh]
		. ../vasp_modify_md.sh

		echo 
		echo == Run [$VASPRunCMD]
		$VASPRunCMD | $TEE $HostName.txt

		echo
		echo == Run [$DataRoot/scripts/post_md.sh]
		$DataRoot/scripts/post_md.sh

		echo
		echo == cp CONTCAR to ../CONTCAR
		cp ../POSCAR ../POSCAR.prev
		cp CONTCAR ../POSCAR

		echo 
		echo == Change dir  to [$basedir]
		cd $basedir
	fi
fi


if [ "$do_relax1" = "1" ]; then
	echo ' '
	echo '================================================================'
	echo "  Run Relax1"
	echo '================================================================'

	workdir=$relax1_dir

	if [ "$SkipIfDirectoryExists" = "1" -a -d $workdir ]; then
		echo "Dir [$workdir] exists."
		echo "Skip."
	else
		echo ' '
		echo '================================================================'
		echo "  VCRelax1 run"
		echo '================================================================'
		echo 

		echo 
		echo == Change dir  to [$basedir]
		cd $basedir

		echo == Make dir [$workdir]

		mkdir $workdir

		echo 
		echo == cp files to [$workdir]
		cp ./{INCAR,POSCAR,POTCAR,KPOINTS} $workdir
#		cp ./{*.sh,INCAR,POSCAR,POTCAR,KPOINTS} $workdir

		echo 
		echo == Change dir to [$workdir]
		cd $workdir

		echo
		echo == Run [../vasp_modify_relax1.sh]
		. ../vasp_modify_relax1.sh

		echo 
		echo == Run [$VASPRunCMD]
		$VASPRunCMD | $TEE $HostName.txt

		echo
		echo == Run [$DataRoot/scripts/post_relax.sh]
		$DataRoot/scripts/post_relax.sh

		echo
		echo == cp CONTCAR to ../CONTCAR
		cp ../POSCAR ../POSCAR.prev
		cp CONTCAR ../POSCAR

		echo 
		echo == Change dir  to [$basedir]
		cd $basedir
	fi
fi


if [ "$do_relax2" = "1" ]; then
	echo ' '
	echo '================================================================'
	echo "  Run Relax2"
	echo '================================================================'

	workdir=$relax2_dir

	if [ "$SkipIfDirectoryExists" = "1" -a -d $workdir ]; then
		echo "Dir [$workdir] exists."
		echo "Skip."
	else
		echo ' '
		echo '================================================================'
		echo "  VCRelax2 run"
		echo '================================================================'
		echo 

		echo 
		echo == Change dir  to [$basedir]
		cd $basedir

		echo == Make dir [$workdir]

		mkdir $workdir

		echo 
		echo == cp files to [$workdir]
		cp ./{INCAR,POSCAR,POTCAR,KPOINTS} $workdir
#		cp ./{*.sh,INCAR,POSCAR,POTCAR,KPOINTS} $workdir

		echo 
		echo == Change dir to [$workdir]
		cd $workdir

		echo
		echo == Run [../vasp_modify_relax2.sh]
		. ../vasp_modify_relax2.sh

		echo 
		echo == Run [$VASPRunCMD]
		$VASPRunCMD | $TEE $HostName.txt

		echo
		echo == Run [$DataRoot/scripts/post_relax.sh]
		$DataRoot/scripts/post_relax.sh

		echo
		echo == cp CONTCAR to ../CONTCAR
		cp ../POSCAR ../POSCAR.prev
		cp CONTCAR ../POSCAR

		echo 
		echo == Change dir  to [$basedir]
		cd $basedir
	fi
fi


if [ "$do_scf" = "1" ]; then
	echo ' '
	echo '================================================================'
	echo "  Run DFT SCF"
	echo '================================================================'

	workdir=$scf_dir

	if [ "$SkipIfDirectoryExists" = "1" -a -d $workdir ]; then
		echo "Dir [$workdir] exists."
		echo "Skip."
	else
		echo 
		echo == Change dir  to [$basedir]
		cd $basedir

		echo 
		echo == Make dir [$workdir]

		mkdir $workdir

		echo 
		echo == cp files to [$workdir]
		cp ./{INCAR,POSCAR,POTCAR,KPOINTS} $workdir
#		cp ./{*.sh,INCAR,POSCAR,POTCAR,KPOINTS} $workdir

		echo 
		echo == Change dir to [$workdir]
		cd $workdir

		echo
		echo == Run [../vasp_modify_scf.sh]
		. ../vasp_modify_scf.sh

		echo 
		echo == Run [$VASPRunCMD]
		$VASPRunCMD | $TEE $HostName.txt

		echo
		echo == cp CHG, CHGCAR, WAVECAR to ../
		cp CHG CHGCAR WAVECAR ../

		echo
		echo == Run [$DataRoot/scripts/vasp_post_scf.sh]
		$DataRoot/scripts/post_scf_dos.sh

		echo 
		echo == Change dir  to [$basedir]
		cd $basedir
	fi
fi


if [ "$do_hfscf" = "1" ]; then
	echo ' '
	echo '================================================================'
	echo "  Run HF SCF"
	echo '================================================================'

	workdir=$hfscf_dir

	if [ "$SkipIfDirectoryExists" = "1" -a -d $workdir ]; then
		echo "Dir [$workdir] exists."
		echo "Skip."
	else
		echo 
		echo == Change dir  to [$basedir]
		cd $basedir

		echo 
		echo == Make dir [$workdir]

		mkdir $workdir

		echo 
		echo == cp files to [$workdir]
		cp ./{INCAR,POSCAR,POTCAR,KPOINTS,WAVE*,CHG*} $workdir

		echo 
		echo == Change dir to [$workdir]
		cd $workdir

		echo
		echo == Run [../vasp_modify_hfscf.sh]
		. ../vasp_modify_hfscf.sh

		echo 
		echo == Run [$VASPRunCMD]
		$VASPRunCMD | $TEE $HostName.txt

		echo
		echo == cp CHG, CHGCAR, WAVECAR to ../
		cp CHG     ../CHG.$HybridFunctional
		cp CHGCAR  ../CHGCAR.$HybridFunctional
		cp WAVECAR ../WAVECAR.$HybridFunctional

		echo
		echo == Run [$DataRoot/scripts/vasp_post_scf.sh]
		$DataRoot/scripts/post_scf_dos.sh

		echo 
		echo == Change dir  to [$basedir]
		cd $basedir
	fi
fi


if [ "$do_dos" = "1" ]; then
	echo ' '
	echo '================================================================'
	echo "  Run DOS"
	echo '================================================================'

	workdir=$dos_dir

	if [ "$SkipIfDirectoryExists" = "1" -a -d $workdir ]; then
		echo "Dir [$workdir] exists."
		echo "Skip."
	else
		echo 
		echo == Change dir  to [$basedir]
		cd $basedir

		echo 
		echo == Make dir [$workdir]

		mkdir $workdir

		echo 
		echo == cp files [cp ./{INCAR,POSCAR,POTCAR,KPOINTS,CHG,CHGCAR,WAVECAR} $workdir]
		cp ./{INCAR,POSCAR,POTCAR,KPOINTS,CHG,CHGCAR,WAVECAR} $workdir
#		cp ./{*.sh,INCAR,POSCAR,POTCAR,KPOINTS,CHG,CHGCAR,WAVECAR} $workdir

		echo 
		echo == Change dir to [$workdir]
		cd $workdir

		echo
		echo == Run [../vasp_modify_dos.sh]
		. ../vasp_modify_dos.sh

		echo 
		echo == Run [$VASPRunCMD]
		$VASPRunCMD | $TEE $HostName.txt

		echo
		echo == Run [$DataRoot/scripts/post_scf_dos.sh]
		$DataRoot/scripts/post_scf_dos.sh

		echo 
		echo == Change dir  to [$basedir]
		cd $basedir
	fi
fi


if [ "$do_band" = "1" ]; then
	echo ' '
	echo '================================================================'
	echo "  Run Band"
	echo '================================================================'

	workdir=$band_dir

	if [ "$SkipIfDirectoryExists" = "1" -a -d $workdir ]; then
		echo "Dir [$workdir] exists."
		echo "Skip."
	else
		echo 
		echo == Change dir  to [$basedir]
		cd $basedir

		echo 
		echo == Make dir [$workdir]

		mkdir $workdir

		echo 
		echo == cp files to [$workdir]
		cp ./{INCAR,POSCAR,POTCAR,KPOINTS,CHG,CHGCAR,WAVECAR} $workdir
#		cp ./{*.sh,INCAR,POSCAR,POTCAR,KPOINTS,CHG,CHGCAR,WAVECAR} $workdir

		echo 
		echo == Change dir to [$workdir]
		cd $workdir

		echo
		echo == Run [../vasp_modify_band.sh]
		. ../vasp_modify_band.sh

		echo 
		echo == Run [$VASPRunCMD]
		$VASPRunCMD | $TEE $HostName.txt

		echo
		echo == Run [$DataRoot/scripts/post_band.sh]
		$DataRoot/scripts/post_band.sh

		echo 
		echo == Change dir  to [$basedir]
		cd $basedir
	fi
fi


if [ "$do_phonon" = "1" ]; then
	echo ' '
	echo '================================================================'
	echo "  Run Phonon"
	echo '================================================================'

	workdir=$phonon_dir

	if [ "$SkipIfDirectoryExists" = "1" -a -d $workdir ]; then
		echo "Dir [$workdir] exists."
		echo "Skip."
	else
		echo 
		echo == Change dir  to [$basedir]
		cd $basedir

		echo 
		echo == Make dir [$workdir]

		mkdir $workdir

		echo 
		echo == cp files to [$workdir]
		cp ./{INCAR,POSCAR,POTCAR,KPOINTS,CHG,CHGCAR,WAVECAR} $workdir
#		cp ./{*.sh,INCAR,POSCAR,POTCAR,KPOINTS,CHG,CHGCAR,WAVECAR} $workdir

		echo 
		echo == Change dir to [$workdir]
		cd $workdir

		echo
		echo == Run [../vasp_modify_phonon.sh]
		. ../vasp_modify_phonon.sh

		echo 
		echo == Run [$VASPRunCMD]
		$VASPRunCMD | $TEE $HostName.txt

		echo
		echo == Run [../vasp_modify_scf_dos.sh]
		../vasp_modify_scf_dos.sh

		echo 
		echo == Change dir  to [$basedir]
		cd $basedir
	fi
fi


if [ "$do_phonopy" = "1" ]; then
	echo ' '
	echo '================================================================'
	echo "  Run Phonopy"
	echo '================================================================'

	workdir=phonopy_dir

	if [ "$SkipIfDirectoryExists" = "1" -a -d $workdir ]; then
		echo "Dir [$workdir] exists."
		echo "Skip."
	else
		echo 
		echo == Change dir  to [$basedir]
		cd $basedir

		echo 
		echo == Make dir [$workdir]

		mkdir $workdir

		echo 
		echo == cp files to [$workdir]
		cp ./{INCAR,POSCAR,POTCAR,KPOINTS,CHG,CHGCAR,WAVECAR} $workdir
#		cp ./{*.sh,INCAR,POSCAR,POTCAR,KPOINTS,CHG,CHGCAR,WAVECAR} $workdir

		echo 
		echo == Change dir to [$workdir]
		cd $workdir

		echo
		echo == Run [../vasp_modify_phonopy.sh]
		. ../vasp_modify_phonopy.sh

		BaseDirRegExp=`echo $BaseDir | sed -e "s/\//:_:/g"`

		sed \
		    -e "s/BaseDir={.*$/BaseDir=\"$BaseDirRegExp\"/" \
		    -e "s/^nCPU=.*$/nCPU=$nCPU/" \
		    -e "s/^NPAR=.*$/NPAR=$NPAR/" \
		    -e "s/^KPAR=.*$/KPAR=$KPAR/" \
		    -e "s/^nCPUForMKL=.*$/nCPUForMKL=$nCPUForMKL/" \
		    -e "s/^MKL_NUM_THREADS=.*$/MKL_NUM_THREADS=$MKL_NUM_THREADS/" \
		    -e "s/^Usebsub=.*$/Usebsub=$Usebsub/" \
		    -e "s/^Precision=.*$/Precision=$PrecisionPhonopy/" \
		    -e "s/^ENCUT=.*$/ENCUT=$ENCUTPhonopy/" \
		    -e "s/^EDIFF=.*$/EDIFF=$EDIFF/" \
		    -e "s/^EDIFFG=.*$/EDIFFG=$EDIFFG/" \
		    -e "s/^aKProduct=.*$/aKProduct=$aKProductPhonopy/" \
		    -e "s/^GammaOnly=.*$/GammaOnly=$GammaOnlyPhonopy/" \
		    -e "s/^ISMEAR=.*$/ISMEAR=$ISMEARPhonopy/" \
		    -e "s/^SIGMA=.*$/SIGMA=$SIGMAPhonopy/" \
		    -e "s/^KeepSymmetry=.*$/KeepSymmetry=$KeepSymmetryPhonopy/" \
		    -e "s/^Interface=.*$/Interface=$InterfacePhonopy/" \
		    -e "s/^nSupercell=.*$/nSupercell=\"$nSupercell\"/" \
		    -e "s/^MeshPoints=.*$/MeshPoints=\"$MeshPoints\"/" \
		    -e "s/^BandPoints=.*$/BandPoints=$BandPoints/" \
		    -e "s/^fConversion=.*$/fConversion=$fConversion/" \
		    -e "s/^NonAnalyticalTermCorrection=.*$/NonAnalyticalTermCorrection=$NonAnalyticalTermCorrection/" \
		    -e "s/^Title=.*$/Title=$Title/" \
		    -e "s/^EMail=.*$/EMail=$EMail/" \
		    -e "s/^SMTPServer=.*$/SMTPServer=$SMTPServer/" \
		    -e "s/^Domain=.*$/Domain=$Domain/" \
		    -e "s/^MailerName=.*$/MailerName=$MailerName/" \
		    -e "s/^From=.*$/From=$From/" \
		    -e "s/^DoSendMail=.*$/DoSendMail=$DoSendMail/" \
		    -e "s/^DoPhonopyDOS=.*$/DoPhonopyDOS=$DoPhonopyDOS/" \
		    -e "s/^DoPhonopyThermalProperties=.*$/DoPhonopyThermalProperties=$DoPhonopyThermalProperties/" \
		    -e "s/^DoPhonopyPDOS=.*$/DoPhonopyPDOS=$DoPhonopyPDOS/" \
		    -e "s/^DoPhonopyBand=.*$/DoPhonopyBand=$DoPhonopyBand/" \
		    $DoPhonopyTemplatePath > $DoPhonopyCMD
		chmod +x $DoPhonopyCMD

		echo "Run [$DoPhonopyCMD]."
		$DoPhonopyCMD

		echo "IRREPS = $IRREPS" > irreps.cfg
		echo "SHOW_IRREPS = .TRUE." >> irreps.cfg
		echo "Run [phonopy --dim="$nSupercell" irreps.cfg]"
		phonopy --dim="$nSupercell" irreps.cfg | $TEE irreps.txt

		echo
		echo == Run [$DataRoot/scripts/post_phonopy.sh]
		$DataRoot/scripts/post_phononpy.sh

		echo 
		echo == Change dir  to [$basedir]
		cd $basedir
	fi
fi


if [ "$do_ir" = "1" ]; then
	echo ' '
	echo '================================================================'
	echo "  Run IR"
	echo '================================================================'

	workdir=$ir_dir

	if [ "$SkipIfDirectoryExists" = "1" -a -d $workdir ]; then
		echo "Dir [$workdir] exists."
		echo "Skip."
	else
		echo 
		echo == Change dir  to [$basedir]
		cd $basedir

		echo 
		echo == Make dir [$workdir]

		mkdir $workdir

		echo 
		echo == cp files to [$workdir]
		cp ./$phonon_dir/{INCAR,POSCAR,POTCAR,KPOINTS,OUTCAR} $workdir
#		cp ./{INCAR,POSCAR,POTCAR,KPOINTS,CHG,CHGCAR,WAVECAR} $workdir
#		cp ./{*.sh,INCAR,POSCAR,POTCAR,KPOINTS,CHG,CHGCAR,WAVECAR} $workdir

		echo 
		echo == Change dir to [$workdir]
		cd $workdir

#		echo
#		echo == Run [../vasp_modify_ir.sh]
#		. ../vasp_modify_ir.sh

#		echo 
#		echo == Run [$VASPRunCMD]
#		$VASPRunCMD | $TEE $HostName.txt

		echo 
		echo == Run [$CalIRCMD]
		$CalIRCMD

#		echo
#		echo == Run [../vasp_modify_ir.sh]
#		../vasp_modify_ir.sh

		echo 
		echo == Change dir  to [$basedir]
		cd $basedir
	fi
fi


if [ "$do_raman" = "1" ]; then
	echo ' '
	echo '================================================================'
	echo "  Run Raman"
	echo '================================================================'

	workdir=$raman_dir

	if [ "$SkipIfDirectoryExists" = "1" -a -d $workdir ]; then
		echo "Dir [$workdir] exists."
		echo "Skip."
	else
		echo 
		echo == Change dir  to [$basedir]
		cd $basedir

		echo 
		echo == Make dir [$workdir]

		mkdir $workdir

		echo 
		echo == cp files to [$workdir]
		cp ./{INCAR,POSCAR,POTCAR,KPOINTS,CHG,CHGCAR,WAVECAR} $workdir
#		cp ./{*.sh,INCAR,POSCAR,POTCAR,KPOINTS,CHG,CHGCAR,WAVECAR} $workdir

		echo 
		echo == Change dir to [$workdir]
		cd $workdir

		echo
		echo == Run [../vasp_modify_raman.sh]
		. ../vasp_modify_raman.sh

		echo 
		echo == Run [$VASPRunCMD]
		$VASPRunCMD | $TEE $HostName.txt

#		echo 
#		echo == Run [$CalIRCMD]
#		$CalIRCMD

		echo 
		echo Raman modes [$RamanModes]
		for i in $RamanModes
		do
			export VASP_RAMAN_PARAMS="${i}_${i}_${RamannDeriv}_${RamanStepSize}"
			echo "  Run[$CalRamanCMD] with VASP_RAMAN_PARAMS=$VASP_RAMAN_PARAMS]"
			$CalRamanCMD --gen

			mv POSCAR.-1.out POSCAR.-1.${i}
			rm *.prev *.phonon
		done
		cp POSCAR.phon POSCAR

		for i in $RamanModes
		do
			if [ -f $BackupDir/POSCAR.-1.${i} ]; then
				WorkDir=$BackupDir/$i
				if [ "$SkipIfDirectoryExists" = "1" -a -d $WorkDir ]; then
					echo "Dir [$WorkDir] exists."
					echo "Skip."
				else
					rmdir -rif $WorkDir
					mkdir $WorkDir
					cd $WorkDir
					echo "   Run in [$WorkDir]"

					cp $OriginalDir/Phonon/{INCAR,POSCAR,POTCAR,KPOINTS} .
					cp $OriginalDir/hosts .
					echo "[$CMDRaman $WorkDir/INCAR]"
					$CMDRaman $WorkDir/INCAR
					cp POSCAR POSCAR.phon
					cp $OriginalDir/Phonon/OUTCAR OUTCAR.phon

					export VASP_RAMAN_PARAMS="${i}_${i}_${RamannDeriv}_${RamanStepSize}"
					echo "  Run [$CalRamanCMD] with VASP_RAMAN_PARAMS=$VASP_RAMAN_PARAMS]"
					$CalRamanCMD | $TEE `hostname`-vasp_raman.out.txt

					if [ -s vasp_raman.dat ]; then
						echo ' '
					else
						SendMail "$EMail" "$SMTPServer" "$MailerName" "$Domain" "$From" \
						     "$Title" "$HostName: $Title failed in VASP VCRelaxRough"
						PrintError "[vasp_raman.dat] does not exist." \
						 	  "Something wrong in Raman. Continue." 0
					fi

					rm WAV* CHG* *.prev *.phonon *.prev
				fi
			fi
		done

#		echo
#		echo == Run [../vasp_modify_ir.sh]
#		../vasp_modify_ir.sh

		echo 
		echo == Change dir  to [$basedir]
		cd $basedir
	fi
fi


echo ' '
echo '================================================================'
echo "  Make summary"
echo '================================================================'

echo 
echo == Change dir  to [$basedir]
cd $basedir

echo
echo == Execute [$VASPScriptDir/Summarize2.pl ./ \> Summarize.txt]
perl $VASPScriptDir/Summarize2.pl ./ > Summarize.txt
echo
echo == Execute [$VASPScriptDir/Summarize.pl  ./ \> Summarize-SpinPolarized.txt]
perl $VASPScriptDir/Summarize.pl  ./ > Summarize-SpinPolarized.txt
echo
echo == Execute [$VASPScriptDir/MakeSummaryHTML.pl]
perl $VASPScriptDir/MakeSummaryHTML.pl


echo 
echo ======================================================================
echo $0 finished.
echo ======================================================================
echo 

echo 
echo == Change dir  to [$basedir]
cd $basedir

echo
