import os
import sys
import shutil
import glob
import numpy as np
from numpy import sin, cos, tan, pi
import csv
from pprint import pprint


from tklib.tkfile import tkFile
from tklib.tkutils import terminate, pint, pfloat, getarg, getintarg, getfloatarg
from tklib.tkapplication import tkApplication

from tklib.tkcrystal.tkcif import tkCIF
from tklib.tkcrystal.tkcrystal import tkCrystal
from tklib.tkcrystal.tkatomtype import tkAtomType


debug = 0

infile = 'test.cif'
single = 1
findvalidstructure = 1


def usage():
    print("")
    print("Usage:")
    print(" python {} infile ".format(sys.argv[0]))
    print("     ex: python {} {}"
                .format(sys.argv[0], infile))

def terminate(message = None):
    if message is not None:
        print("")
        print(message)

    print("")
    usage()
    print("")
    
    input("\nPress ENTER to exit>>")
    exit()


infile = getarg(1, infile)
debug  = getintarg(2, debug)


def main():
    app = tkApplication()
    logfile = app.replace_path(infile, template = ["{dirname}", "{filebody}-out.txt"])
    print(f"Open logfile [{logfile}]")
    app.redirect(targets = ["stdout", logfile], mode = 'w')

    print("")
    print("=============== CIF file read ============")
    print("infile: {}".format(infile))
    print("single: {}".format(single))
    print("findvalidstructure: {}".format(findvalidstructure))
    print("debug: {}".format(debug))

    print("")
    print("Read [{}]".format(infile))

#    if not tkutils.IsFile(infile):
#        terminate("Error: Invalid infile [{}]".format(infile))
    cif = tkCIF()
    cif.debug = debug
    
    cifdata = cif.ReadCIF(infile, find_valid_structure = findvalidstructure)
    cif.Close()

    if not cifdata:
        terminate("Error: Could not get cifdat from infile [{}]".format(infile))

    cifdata.Print()
    cry = cifdata.GetCrystal()
    print("")
    print("==============================================")
    cry.PrintInf()
    print("")
    print("==============================================")
            
    d = cry.Density()
    ad = cry.AtomDensity()
    

    terminate()


if __name__ == "__main__":
    main()



"""

print "\n";
#==========================================
# Crystalクラスから、SpaceGroupクラスを取り出す
# この時点で、並進対称要素の抽出、LatticeSystemの抽出は完了している
#==========================================
my $SPG = $Crystal->GetCSpaceGroup();
my $SPGName = $SPG->SPGName();
my $iSPG    = $SPG->iSPG();
my $LatticeSystem = $SPG->LatticeSystem();
print "Space Group: $SPGName ($iSPG)\n";
print "Lattice System: $LatticeSystem\n";

print "\n";
my $nTranslation = $SPG->nTranslation();
print "nTranslation: $nTranslation\n";
for(my $i = 0 ; $i < $nTranslation ; $i++) {
	my ($x,$y,$z) = $SPG->TranslationVector($i+1);
	print "   ($x, $y, $z)\n";
}

print "\n";
my $nSymmetryOperation = $SPG->nSymmetryOperation();
print "nSymmetryOperation: $nSymmetryOperation\n";
for(my $i = 0 ; $i < $nSymmetryOperation ; $i++) {
	my $symop = $SPG->SymmetryOperation($i+1);
	print "   $symop\n";
	
	my ($x1,$y1,$z1,$t1,
		$x2,$y2,$z2,$t2,
		$x3,$y3,$z3,$t3)
		= $SPG->SymmetryOperationByMatrix($i+1);
	print "      $x1 $y1 $z1  $t1\n";
	print "      $x2 $y2 $z2  $t2\n";
	print "      $x3 $y3 $z3  $t3\n";
}

print "\n";
#==========================================
# Crystalクラス中の、原子の種類 AtomTypeクラスのリストをとる
#==========================================
my @AtomTypeList = $Crystal->GetCAtomTypeList();
my $nAtomType = @AtomTypeList;
print "nAtomType: $nAtomType\n";
for(my $i = 0 ; $i < $nAtomType ; $i++) {
	my $label    = $AtomTypeList[$i]->Label();
	my $atomname = $AtomTypeList[$i]->AtomName();
	my $charge   = $AtomTypeList[$i]->Charge();
	my $AtomicNumber = $AtomTypeList[$i]->AtomicNumber();
	my $AtomicMass   = $AtomTypeList[$i]->AtomicMass();
	my $i1 = $i+1;
	print "   #$i1: $label : $atomname [$AtomicNumber] ($charge) "
		."($AtomicMass)\n";
}

print "\n";
#==========================================
# Crystalクラス中の、非対称単位中の原子 AsymmetricAtomSiteクラスのリストをとる
#==========================================
my @AtomSiteList        = $Crystal->GetCAsymmetricAtomSiteList();
my $nAsymmetricAtomSite = @AtomSiteList;
print "nAsymmetricAtomSite: $nAsymmetricAtomSite\n";
for(my $i = 0 ; $i < $nAsymmetricAtomSite ; $i++) {
	my $label     = $AtomSiteList[$i]->Label();
	my $type      = $AtomSiteList[$i]->AtomName();
	my ($x,$y,$z) = $AtomSiteList[$i]->Position(1);
	my $occupancy = $AtomSiteList[$i]->Occupancy();
	my $iAtomType = $Crystal->FindiAtomType($type);

	print "   $label ($type)[#$iAtomType]: ($x, $y, $z) [$occupancy]\n";
}

#==========================================
# 対称要素を使い、非対称単位中の原子を展開する
# この際、Densityも計算される
#==========================================
$Crystal->ExpandCoordinates();

print "\n";
my @ExpandedAtomSiteList   = $Crystal->GetCExpandedAtomSiteList();
my $nTotalExpandedAtomSite = $Crystal->nTotalExpandedAtomSite();
my @nMultiplicityExpandedAtomSiteList 
	= $Crystal->GetCnMultiplicityExpandedAtomSiteList();
print "nTotalExpandedAtomSite: $nTotalExpandedAtomSite\n";
for(my $i = 0 ; $i < $nTotalExpandedAtomSite ; $i++) {
	my $label     = $ExpandedAtomSiteList[$i]->Label();
	my $type      = $ExpandedAtomSiteList[$i]->AtomName();
	my ($x,$y,$z) = $ExpandedAtomSiteList[$i]->Position(1);
	my $occupancy = $ExpandedAtomSiteList[$i]->Occupancy();
	my $id = $ExpandedAtomSiteList[$i]->IdAsymmetricAtomSite();
	my $mult = $nMultiplicityExpandedAtomSiteList[$id-1];
	my $i1 = $i+1;
	print "   $i1: [$id]$label ($type): ($x, $y, $z) [$occupancy][$mult]\n";
}

print "\n";
my $Density = $Crystal->Density();
print "Density: $Density g/cm^3\n";

#==========================================
# $Crystalをもとに、超格子の結晶格子$SuperLatticeを作る
#==========================================
my ($nx,$ny,$nz) = (2,2,2);
$Crystal->CreateSuperLattice($nx,$ny,$nz);
my $SuperLattice = $Crystal->GetCSuperLattice();

print "\n";
print "SuplaerLattice $nx x $ny x $nz\n";
my ($a,$b,$c,$alpha,$beta,$gamma) = $SuperLattice->LatticeParameters();
print "Cell: $a $b $c  $alpha $beta $gamma\n";

my @ExpandedAtomSiteList = $SuperLattice->GetCExpandedAtomSiteList();
my $nTotalExpandedAtomSite = $SuperLattice->nTotalExpandedAtomSite();
my @nMultiplicityExpandedAtomSiteList 
	= $SuperLattice->GetCnMultiplicityExpandedAtomSiteList();
print "nTotalExpandedAtomSite: $nTotalExpandedAtomSite\n";
for(my $i = 0 ; $i < $nTotalExpandedAtomSite ; $i++) {
	my $label     = $ExpandedAtomSiteList[$i]->Label();
	my $type      = $ExpandedAtomSiteList[$i]->AtomName();
	my ($x,$y,$z) = $ExpandedAtomSiteList[$i]->Position(1);
	my $occupancy = $ExpandedAtomSiteList[$i]->Occupancy();
	my $id = $ExpandedAtomSiteList[$i]->IdAsymmetricAtomSite();
	my $mult = $nMultiplicityExpandedAtomSiteList[$id-1];
	my $i1 = $i+1;
	print "   $i1: [$id]$label ($type): ($x, $y, $z) [$occupancy][$mult]\n";
}



exit;
"""
