import os
import sys
import re
import numpy as np
from numpy import sqrt, exp, sin, cos, tan, log, log10, pi
import numpy.linalg as LA 
import csv
from matplotlib import pyplot as plt


sys.path.append("c:/Programs/python/lib")
sys.path.append("d:/Programs/python/lib")

from tkargs import tkArgs
from tkutils import pint, pfloat, getarg, getintarg, getfloatarg, IsFile, IsDir
from tkutils import terminate
from tksci.tkoptimize import tkOptimize, ReduceOptParams, RecoverParams
from tkgraphic.tkplot import tkPlot


"""
Fitting temperature dependence of Hall effect to 2 layers model
"""

#===================================
# physical constants
#===================================
pi   = 3.14159265358979323846
pi2  = 2.0 * pi
h    = 6.6260755e-34    # Js";
hbar = 1.05459e-34      # "Js";
c    = 2.99792458e8     # m/s";
e    = 1.60218e-19      # C";
e0   = 8.854418782e-12; # C<sup>2</sup>N<sup>-1</sup>m<sup>-2</sup>";
kB   = 1.380658e-23     # JK<sup>-1</sup>";
me   = 9.1093897e-31    # kg";
R    = 8.314462618      # J/K/mol
a0   = 5.29177e-11      # m";


# meff in me0, Nc in cm^{-3}
def meff2Nc(meff, T):
    return 2.0 * pow(2.0 * pi * meff * me * kB * T / h / h, 1.5) * 1.0e-6    # cm-3

# meff in me0, Dc0 in cm^{-3}eV^{-1.5}
def meff2Dc0(meff):
    return 2.0 * 2.0 * pi * pow(2.0 * m * me, 1.5) / h / h / h * 1.0e-6 * pow(e, 1.5)

# Nc in cm^{-3}, meff in me0
def Nc2meff(Nc, T):
#NC = 2 * (2 pi m* kT / h / h)^1.5
    meff = Nc * 1.0e6 / 2.0 / pow(2.0 * pi * kB * T / h / h, 1.5)
    meff = pow(meff, 2.0 / 3.0) / me
    return meff

m = 0.35
T = 300.0
#Nc = 2.0 * pow(2.0 * pi * m * me * kB * T / h / h, 1.5) * 1.0e-6    # cm-3
Nc = meff2Nc(m, T)
#Dc0 = 2.0 * 2.0 * pi * pow(2.0 * m * me, 1.5) / h / h / h * 1.0e-6 * pow(e, 1.5)
Dc0 = meff2Dc0(m)
print("Nc=", Nc)
print("Dc0=", Dc0)


meff = Nc2meff(Nc, T)
Dc0  = meff2Dc0(meff)
print("Nc=", Nc, meff, Dc0)
