#間違っていても責任はとりません、野元
#Single parabric band modelを使ってLorentz数、Keを計算します。
# -*- coding: utf-8 -*- 

import os
import numpy as np
import scipy.integrate as integrate
import sys


#Constant value
kB = 8.6173303e-5 #(kB/e)
kb = 1.38065E-23
evalue = 1.60218E-19
h  = 6.63E-34
me = 9.11E-31
pi = 3.141592654


nmaxiter = 100
eps = 1.0e-10
a =  1000 #max
b = -100 #min

T     = float(sys.argv[1])
S     = float(sys.argv[2]) * 1.0e-6  # V/K
S = abs(S)
sigma = float(sys.argv[3]) * 1.0e2   # S/m
kappa = float(sys.argv[4])           # W/m/K

print(f"T     = {T} K")
print(f"S     = {S} V/K")
print(f"sigma = {sigma} S/m")
print(f"kappa = {kappa} W/m/K")

#Calculation method
def ke(sigma, L, T):
    return sigma * L * T

def Fr(EF, r):
    result = integrate.quad(lambda E: E**r / (1 + np.exp(E - EF)), 0, np.inf)
    #print("felmi_integral = {}".format(result))
    fel = float(result[0])
    #print(fel)
    return fel

def S_comprison(EF, S):
    diff = S - kB * (2.0 * Fr(EF, r = 1.0) / Fr(EF, r = 0.0) - EF)
    #print("diff = {}".format(diff))
    return diff

def Lorenz(EF):
    answer = kB**2 * (3.0 * Fr(EF, r = 2.0) / Fr(EF, r = 0.0) - (2.0 * Fr(EF, r = 1.0) / Fr(EF, r = 0.0))**2)
    return answer

def bisection(max, min, err):
    EF = (max + min) / 2.0
    ymin = S_comprison(min, S)
    ymax = S_comprison(max, S)
    yh   = S_comprison(EF,  S)
    print("EF,dS=", EF, ymin, ymax, yh, min, max)

    if ymin * ymax > 0.0:
        print("Error: ymin * ymax > 0.0")
        exit()

    for i in range(nmaxiter):
        print(f"EF={EF:8.3f} d=({ymin:8.4g} {yh:8.4g} {ymax:8.4g}) min={min:8.3f} max={max:8.3f}")

        if abs(yh) < eps:
#        if abs(yh) < eps or abs(max - min) < eps:
            return EF
        
        if ymin * yh > 0.0:
            min = EF
        else:
            max = EF

        EF = (max + min) / 2.0
        ymin = S_comprison(min, S)
        ymax = S_comprison(max, S)
        yh   = S_comprison(EF,  S)

    else:
        print(f"Did not converge in {nmaxiter} iterations.")
        exit()
    
#Execution
EF = bisection(max = a, min = b, err = 1e-10)
print(f"EF/kT = {EF}")

L = Lorenz(EF)
ke = ke(sigma, L, T)
print(f"L  = {L}")
print(f"ke = {ke} W/m/K")

outfile = 'out.csv'
is_exist = os.path.exists(outfile)
fp = open(outfile, 'a')
if not is_exist:
    fp.write("T(K),S (V/K),sigma (S/m),kappa (W/mK),EF/kT,L,ke\n")
fp.write(f"{T},{S},{sigma},{kappa},{EF},{L},{ke}\n")
fp.close()
