import os
import sys


#================================
# global parameters
#================================
plugin_ver = "data:0.1"  # data_list_type = [x, y]

default_ext = ".csv"
input_type  = "CSV"
output_type = "pXRD"

class tkParams():
    pass

cparams = tkParams()


def initialize(app = None, cparams = None):
    cparams.infile = f'*{default_ext}'
    cparams.xmin  = 20.0
    cparams.xmax  = 120.0
    cparams.xstep = 0.02

def update_vars(app = None, cparams = None):
    argv = sys.argv
    narg = len(argv)
    
    if narg < 1:
        exit()

    if narg >= 1:
        cparams.infile = argv[0]
    if narg >= 2:
        cparams.xmin =float(argv[1])
    if narg >= 3:
        cparams.xmax =float(argv[2])
    if narg >= 4:
        cparams.xstep =float(argv[3])

def check_file_type(infile, inf = None, app = None, cparams = None):
    if not os.path.isfile(infile):
        return f"Error: file [{infile}] does not exist"

    header, ext = os.path.splitext(infile)
    if ext.lower() != default_ext:
        return None

    return {"file_type": "CSV"}

def read_data(infile, app = None, cparams = None, print_level = 0):
    if print_level:
        print("")
        print(f"Read [{infile}]")

    if not os.path.isfile(infile):
        app.terminate(f"Error: Can not read [{infile}]")

    fp = open(cparams.infile, 'r', encoding = 'UTF-8')
    if not fp:
        print(f"Error: Cannot read [{cparams.infile}]")
        exit()

    line = fp.readline()
    a = line.split(',')
    labels = [a[0].strip(), a[1].strip()]

    x = []
    y = []
    while 1:
        line = fp.readline()
        if not line:
            break
        
        a = line.split(',')
        x.append(float(a[0]))
        y.append(float(a[1]))

    fp.close()

    basename    = os.path.basename(cparams.infile)
    dirname     = os.path.dirname(cparams.infile)
    header, ext = os.path.splitext(basename)
    filebody    = header
    inf = {}
    inf["data_list_type"] = "[x, y]"
    inf["filename"]    = infile
    inf["sample_name"] = basename
    inf["yscale"]      = 'linear'
    inf["meta"]        = []
    inf["data_list"]   = [x, y]
    inf["nregion"]     = 1
    inf["nspectrum"]   = 1
    inf["ndata"]       = len(x)
    inf["xmin"]        = x[0]
    inf["xmax"]        = x[inf["ndata"] - 1]
    inf["xstep"]       = x[1] - x[0]
    inf["labels"]      = labels

    return inf

def print_data(inf, app = None, cparams = None):
    print("")
    print("filename :", inf["filename"])
    print("sample   :", inf["sample_name"])
    print("yscale   :", inf["yscale"])
    print("labels   :", inf["labels"])
    print("nspectrum:", inf["nspectrum"])
    print("ndata    :", inf["ndata"])
    print(f"2Q range : {inf['xmin']} - {inf['xmax']} degrees, {{inf['xstep']}} step")
    
    nspectrum = inf["nspectrum"]
    data_list = inf["data_list"]
    print("\t".join(inf["labels"]))
    for i in range(inf["ndata"]):
        v = [f"{data_list[ispectrum][i]}" for ispectrum in range(nspectrum + 1)]
        print("\t".join(v))

def main():
    print("")
    print( "==========================================================================")
    print(" Convert CIF file to powder XRD pattern")
    print( "==========================================================================")
    print(f"Input file: {cparams.infile}")

    file_type = check_file_type(cparams.infile)
    if file_type is None:
        app.terminate(f"Error: [{cparams.infile}] invalid file type", usage = usage)
    if 'Error' in file_type:
        app.terminate(file_type, usage = usage)

    inf = read_data(cparams.infile, cparams = cparams)
    print_data(inf)


if __name__ == "__main__":
    main()
